#! /usr/bin/env python

# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import logging
import warnings
logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)

import argparse
import subprocess
import socket
import os
import shutil
from glob import glob

import numpy as np
import math
import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt

import pesummary
from pesummary.webpage import webpage
from pesummary.utils import utils
from pesummary.plot import plot
from pesummary.one_format.data_format import one_format

import h5py

import copy

import lal
import lalsimulation as lalsim

__doc__ == "Parameters to run post_processing.py from the command line"


def command_line():
    """Creates an ArgumentParser object which holds all of the information
    from the command line.
    """
    parser = argparse.ArgumentParser(description=__doc__)
    parser.add_argument("-w", "--webdir", dest="webdir",
                        help="make page and plots in DIR", metavar="DIR")
    parser.add_argument("-b", "--baseurl", dest="baseurl",
                        help="make the page at this url", metavar="DIR",
                        default=None)
    parser.add_argument("-s", "--samples", dest="samples",
                        help="Posterior samples hdf5 file", nargs='+',
                        default=None)
    parser.add_argument("-a", "--approximant", dest="approximant",
                        help="waveform approximant used to generate samples",
                        nargs='+', default=None)
    parser.add_argument("--email", action="store",
                        help="send an e-mail to the given address with a link to the finished page.",
                        default=None, metavar="user@ligo.org")
    parser.add_argument("--dump", action="store_true",
                        help="dump all information onto a single html page",
                        default=False)
    parser.add_argument("-c", "--config", dest="config",
                        help="configuration file associcated with each samples file.", nargs='+',
                        default=None)
    parser.add_argument("--sensitivity", action="store_true",
                        help="generate sky sensitivities for HL, HLV",
                        default=False)
    parser.add_argument("--add_to_existing", action="store_true",
                        help="add new results to an existing html page",
                        default=False)
    parser.add_argument("-e", "--existing_webdir", dest="existing",
                        help="web directory of existing output",
                        default=None)
    parser.add_argument("-i", "--inj_file", dest="inj_file",
                        help="path to injetcion file", nargs='+',
                        default=None)
    parser.add_argument("--user", dest="user", help=argparse.SUPPRESS,
                        default="albert.einstein")
    return parser

def convert_to_standard_format(samples, injections):
    """Convert the input files to the standard format

    Parameters
    ----------
    samples: str/list
        either a string or a list of strings giving the path to the input
        samples file
    """
    if not injections:
        injections = [None]*len(samples)
    for num, i in enumerate(samples):
        opts.samples[num] = one_format(i, injections[num])

def run_checks(opts):
    """Check the command line inputs

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    """
    # check the command line arguments
    if opts.webdir:
        # make the web directory
        utils.make_dir(opts.webdir)
        if opts.samples:
            pass
        else:
            raise Exception("Please run python main.py --samples [results.hdf] "
                            "--config [config.ini]")
    if opts.approximant == None:
        logging.info("No approximant is given. Trying to extract from "
                     "results file")
        opts.approximant = []
        for i in opts.samples:
            f = h5py.File(i, "r")
            approx = f["approximant"][0]
            if approx == b"none":
                raise Exception("Failed to extract approximant from your results "
                                "file: %s. Please pass the approximant with the flag "
                                "--approximant" %(i.split("_temp")[0]))
            opts.approximant.append(approx.decode("utf-8"))
    if len(opts.samples) != len(opts.approximant):
        raise Exception("The number of results files does not match the number of "
                        "approximants")
    # check that if add_to_existing is specified then existing html page
    # is also given
    if opts.add_to_existing and opts.existing == None:
        raise Exception("Please provide a current html page that you wish "
                        "to add content to")
    if not opts.add_to_existing and opts.existing:
        opts.add_to_existing = True
        logging.info("Existing html page has been given without specifying "
                     "--add_to_existing flag. This is probably and error and so "
                     "manually adding --add_to_existing flag")
    # make relevant directories
    dirs = ["samples", "plots", "js", "html", "css", "plots/corner", "config"]
    if opts.webdir:
        for i in dirs:
            utils.make_dir(opts.webdir + "/{}".format(i))
    if opts.existing:
        # check to see if the existing directory actually exists
        if not os.path.isdir(opts.existing):
            raise Exception("The directory %s does not exist" %(opts.existing))
        # check to see if the given existing directory is the base directory
        entries = glob(opts.existing+"/*")
        if "%s/home.html" %(opts.existing) not in entries:
            raise Exception("Please give the base directory of an existing "
                            "output")
        opts.webdir = opts.existing
    # check to see if webdir exists
    if not os.path.isdir(opts.webdir):
        logging.info("Given web directory does not exist. Creating it now")
        utils.make_dir(opts.webdir)
    # check that number of samples matches number of approximants
    if len(opts.samples) != len(opts.approximant):
        raise Exception("Ensure that the number of approximants match the "
                        "number of samples files")
    # check that numer of samples matches number of config files
    if opts.config and len(opts.samples) != len(opts.config):
        raise Exception("Ensure that the number of samples files match the "
                        "number of config files")
    if opts.inj_file and len(opts.inj_file) != len(opts.samples):
        raise Exception("Ensure that the number of samples matct the number of "
                        "injection files")
    for num, i in enumerate(opts.samples):
        if os.path.isfile(i) == False:
            raise Exception("File %s does not exist" %(i))
        proposed_file = opts.webdir+"/samples/"+opts.approximant[num]+"_"+i.split("/")[-1]
        if os.path.isfile(proposed_file):
            raise Exception("File %s already exists under the name %s. Have "
                            "you already generated a summary page with this "
                            "file?" %(i, proposed_file))
    if opts.add_to_existing and opts.existing:
        if opts.config:
            for i in glob(opts.existing+"/config/*"):
                opts.config.append(i)
        for i in glob(opts.existing+"/html/*_corner.html"):
            example_file = i.split("/")[-1]
            opts.approximant.append(example_file.split("_corner.html")[0])
    # check to see if baseurl is provided. If not guess what it could be
    if opts.baseurl == None:
        try:
            user = os.environ["USER"]
            opts.user = user
        except Exception as e:
            logging.info("Failed to grab user information because %s. " 
                         "Default will be used" %(e))
            user = opts.user
        host = socket.getfqdn()
        opts.baseurl = utils.guess_url(opts.webdir, host, user)
        logging.info("No url is provided. The url %s will be used" %(opts.baseurl))

def copy_files(opts):
    """Copy over files to the web directory

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    """
    # copy over the javascript scripts
    path = pesummary.__file__[:-12]
    scripts = ["search.js", "combine_corner.js", "grab.js", "multi_dropbar.js",
               "multiple_posteriors.js", "side_bar.js", "modal.js"]
    for i in scripts:
        shutil.copyfile(path+"/js/%s" %(i), opts.webdir+"/js/%s" %(i))
    # copy over the css scripts
    scripts = ["image_styles.css", "side_bar.css"]
    for i in scripts:
        shutil.copyfile(path+"/css/%s" %(i), opts.webdir+"/css/%s" %(i))
    # copy over the config file
    if opts.config:
        for num, i in enumerate(opts.config):
            if opts.webdir not in i:
                shutil.copyfile(i, opts.webdir+"/config/"+opts.approximant[num]+"_"+i.split("/")[-1])
    for num, i in enumerate(opts.samples):
        if opts.webdir not in i:
            shutil.copyfile(i, opts.webdir+"/samples/"+opts.approximant[num]+"_"+i.split("/")[-1])

def email_notify(address, path):
    """Send an email to notify the user that their output page is generated.

    Parameters
    ----------
    address: str
        email address that you want the output page to be emailed to
    path:str
        path to the directory where the html page will be generated
    """
    logging.info("Sending email to %s" %(address))
    user = opts.user
    host = socket.getfqdn()
    from_address = "{}@{}".format(user, host)
    subject = "Output page available at {}".format(host)
    message = "Hi {},\n\nYour output page is ready on {}. You can view the result at {}\n".format(user, host, path)
    cmd = 'echo -e "%s" | mail -s "%s" "%s"' %(message, subject, address)
    ess = subprocess.Popen(cmd, shell=True)
    ess.wait()

def _grab_parameters(results):
    """Grab the list of parameters that the sampler varies over

    Parameters
    ----------
    results: str
        string to the results file
    """
    # grab the parameters from the samples
    f = h5py.File(results, "r")
    parameters = [i for i in f["parameter_names"]]
    f.close()                   
    return parameters

def _grab_detectors(params):
    """Grab the detectors from the list of parameters

    Parameters
    ----------
    params: list
        list of parameters that have posterior samples
    """
    detectors = []
    for i in params:
        if b"optimal_snr" in  i:
            detectors.append(i.split(b"_optimal_snr")[0])
    return detectors

def _grab_injection_parameters(results):
    """Grab the injection parameters and injection values

    Parameters
    ----------
    results: str
        string to the results file
    """
    # grab the injection parameters
    f = h5py.File(results, "r")
    inj_par = [i for i in f["injection_parameters"]]
    inj_data = [i for i in f["injection_data"]]
    my_dict = {i:j for i,j in zip(inj_par, inj_data)}
    return my_dict

def _grab_key_data(samples, logL, same_parameters, parameters):
    """Grab the key data for each parameter in the samples file.

    Parameters
    ----------
    samples: list
        list of samples you wish to include
    logL: list
        list of likelihoods for each sample
    parameters: list
        list of parameters that the sampler varies over
    """
    data = {}
    for i in same_parameters:
        indices = parameters.index(b"%s" %(i))
        subset = [j[indices] for j in samples]
        data[i] = {"mean": np.mean(subset),
                   "median": np.median(subset),
                   "maxL": subset[logL.index(np.max(logL))],
                   "std": np.std(subset)}
    return data

def make_plots(opts, colors=None):
    """Generate the posterior sample plots

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    colors: list
        list of colors in hexadecimal format for the different approximants
    """
    latex_labels={b"luminosity_distance": r"$d_{L} [Mpc]$",
                  b"geocent_time": r"$t_{c} [s]$",
                  b"dec": r"$\delta [rad]$",
                  b"ra": r"$\alpha [rad]$",
                  b"a_1": r"$a_{1}$",
                  b"a_2": r"$a_{2}$",
                  b"phi_jl": r"$\phi_{JL} [rad]$",
                  b"phase": r"$\phi [rad]$",
                  b"psi": r"$\Psi [rad]$",
                  b"iota": r"$\iota [rad]$",
                  b"tilt_1": r"$\theta_{1} [rad]$",
                  b"tilt_2": r"$\theta_{2} [rad]$",
                  b"phi_12": r"$\phi_{12} [rad]$",
                  b"mass_2": r"$m_{2} [M_{\odot}]$",
                  b"mass_1": r"$m_{1} [M_{\odot}]$",
                  b"total_mass": r"$M [M_{\odot}]$",
                  b"chirp_mass": r"$\mathcal{M} [M_{\odot}]$",
                  b"log_likelihood": r"$\log{\mathcal{L}}$",
                  b"H1_matched_filter_snr": r"$\rho^{H}_{mf}$",
                  b"L1_matched_filter_snr": r"$\rho^{L}_{mf}$",
                  b"H1_optimal_snr": r"$\rho^{H}_{opt}$",
                  b"L1_optimal_snr": r"$\rho^{L}_{opt}$",
                  b"V1_optimal_snr": r"$\rho^{V}_{opt}$",
                  b"E1_optimal_snr": r"$\rho^{E}_{opts}$",
                  b"spin_1x": r"$S_{1}x$",
                  b"spin_1y": r"$S_{1}y$",
                  b"spin_1z": r"$S_{1}z$",
                  b"spin_2x": r"$S_{2}x$",
                  b"spin_2y": r"$S_{2}y$",
                  b"spin_2z": r"$S_{2}z$",
                  b"chi_p": r"$\chi_{p}$",
                  b"chi_eff": r"$\chi_{eff}$",
                  b"mass_ratio": r"$q$",
                  b"symmetric_mass_ratio": r"$\eta$",
                  b"phi_1": r"$\phi_{1} [rad]$",
                  b"phi_2": r"$\phi_{2} [rad]$",
                  b"cos_tilt_1": r"$\cos{\theta_{1}}$",
                  b"cos_tilt_2": r"$\cos{\theta_{2}}$",
                  b"redshift": r"$z$",
                  b"comoving_distance": r"$d_{com} [Mpc]$",
                  b"mass_1_source": r"$m_{1}^{source} [M_{\odot}]$",
                  b"mass_2_source": r"$m_{2}^{source} [M_{\odot}]$",
                  b"chirp_mass_source": r"$\mathcal{M}^{source} [M_{\odot}]$",
                  b"total_mass_source": r"$M^{source} [M_{\odot}]$",
                  b"cos_iota": r"$\cos{\iota}$"}
    # generate array of both samples
    combined_samples = []
    combined_maxL = []
    # get the parameter names
    parameters = [_grab_parameters(i) for i in opts.samples]
    # grab injection parameters
    injection_parameters = [_grab_injection_parameters(i) for i in opts.samples]
    ind_ra_list, ind_dec_list = [], []
    # generate the individual plots
    for num, i in enumerate(opts.samples):
        approx = opts.approximant[num]
        logging.info("Starting to generate plots for %s" %(approx))
        with h5py.File(i) as f:
            params = [j for j in f["parameter_names"]]
            index = params.index(b"log_likelihood")
            samples = [j for j in f["samples"]]
            likelihood = [j[index] for j in samples]
            f.close()
        data = _grab_key_data(samples, likelihood, parameters[num], parameters[num])
        maxL_params = {j: data[j]["maxL"] for j in parameters[num]}
        maxL_params["approximant"] = approx
        if not opts.existing:
            combined_samples.append(samples)
            combined_maxL.append(maxL_params)
        try:
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                fig = plot._make_corner_plot(samples, parameters[num], latex_labels)
                plt.savefig("%s/plots/corner/%s_all_density_plots.png" %(opts.webdir, approx))
                plt.close()
        except Exception as e:
            logging.info("Failed to generate corner plot because %s" %(e))
        try:
            ind_ra = parameters[num].index(b"ra")
            ind_dec = parameters[num].index(b"dec")
            ind_ra_list.append(ind_ra)
            ind_dec_list.append(ind_dec)
            ra = [j[ind_ra] for j in samples]
            dec = [j[ind_dec] for j in samples]
            fig = plot._sky_map_plot(ra, dec)
            plt.savefig("%s/plots/%s_skymap.png" %(opts.webdir, approx))
            plt.close()
        except Exception as e:
            logging.info("Failed to generate skymap because %s" %(e))
        try:
            detectors = _grab_detectors(parameters[num])
            fig = plot._waveform_plot(detectors, maxL_params)
            plt.savefig("%s/plots/%s_waveform.png" %(opts.webdir, approx))
            plt.close()
        except Exception as e:
            logging.info("Failed to generate waveform plot because %s" %(e))
        for ind, j in enumerate(parameters[num]):
            index = parameters[num].index(b"%s" %(j))
            inj_value = injection_parameters[num][b"%s" %(j)]
            if math.isnan(inj_value):
               inj_value = None 
            param_samples = [k[index] for k in samples]
            fig = plot._1d_histogram_plot(j, param_samples, latex_labels[j], inj_value)
            plt.savefig("%s/plots/1d_posterior_%s_%s.png" %(opts.webdir, approx, j.decode("utf-8")))
            plt.close()
        if opts.sensitivity:
            fig = plot._sky_sensitivity(["H1", "L1"], 0.2,
                                        combined_maxL[num])
            plt.savefig("%s/plots/%s_sky_sensitivity_HL" %(opts.webdir, approx))
            plt.close()
            fig = plot._sky_sensitivity(["H1", "L1", "V1"], 0.2,
                                        combined_maxL[num])
            plt.savefig("%s/plots/%s_sky_sensitivity_HLV" %(opts.webdir, approx))
            plt.close()
    # open the new results file and store the data
    if opts.add_to_existing and opts.existing:
        for i in glob(opts.existing+"/samples/*"):
            opts.samples.append(i)
        parameters = [_grab_parameters(i) for i in opts.samples]
        try:
            for i in parameters:
                ind_ra_list.append(i.index(b"ra"))
                ind_dec_list.append(i.index(b"dec"))
        except:
            pass
        for num, i in enumerate(opts.samples):
            with h5py.File(i) as f:
                params = [j for j in f["parameter_names"]]
                index = params.index(b"log_likelihood")
                samples = [j for j in f["samples"]]
                likelihood = [j[index] for j in samples]
                f.close()
            combined_samples.append(samples)
            data = _grab_key_data(samples, likelihood, parameters[num], parameters[num])
            maxL_params = {j: data[j]["maxL"] for j in parameters[num]}
            if opts.existing in i:
                approx = i.split("/")[-1].split("_")[0]
            else:
                approx = opts.approximant[num]
            maxL_params["approximant"] = approx
            combined_maxL.append(maxL_params)
    # if len(approximants) > 1, then we need to do comparison plots
    if len(opts.approximant) > 1:
        same_parameters = list(set.intersection(*[set(l) for l in parameters]))
        for ind, j in enumerate(same_parameters):
            indices = [k.index(b"%s" %(j)) for k in parameters]
            param_samples = [[k[indices[num]] for k in l] for num,l in enumerate(combined_samples)]
            fig = plot._1d_comparison_histogram_plot(j, opts.approximant,
                                                     param_samples, colors,
                                                     latex_labels[j])
            plt.savefig("%s/plots/combined_posterior_%s" %(opts.webdir, j.decode("utf-8")))
            plt.close()
        try:
            ra_list = [[k[ind_ra_list[num]] for k in l] for num, l in enumerate(combined_samples)]
            dec_list = [[k[ind_dec_list[num]] for k in l] for num, l in enumerate(combined_samples)]
            fig = plot._sky_map_comparison_plot(ra_list, dec_list, opts.approximant,
                                                colors)
            plt.savefig("%s/plots/combined_skymap.png" %(opts.webdir))
            plt.close()
        except Exception as e:
            logging.info("Failed to generate comparison skymap because %s" %(e))
        try:
            fig = plot._waveform_comparison_plot(combined_maxL, colors)
            plt.savefig("%s/plots/compare_waveforms.png" %(opts.webdir))
            plt.close()
        except Exception as e:
            logging.info("Failed to generate waveform comparison plot because "
                         "%s" %(e))

def make_navbar_links(parameters):
    """Generate the links for the navbar

    Parameters
    ----------
    parameters: list
        list of parameters that were used in your study
    """
    links = ["1d_histograms", ["multiple"]]
    if any(b"mass" in s for s in parameters):
        condition = lambda i: True if b"mass" in i and b"source" not in i or b"q" in i \
                              or b"symmetric_mass_ratio" in i else False
        links.append(["masses", [i.decode("utf-8") for i in parameters if condition(i)]])
    if any(b"a_1" in s for s in parameters):
        condition = lambda i: True if b"spin" in i or b"chi_p" in i \
                              or b"chi_eff" in i or b"a_1" in i or b"a_2" in i \
                              else False
        links.append(["spins", [i.decode("utf-8") for i in parameters if condition(i)]])
    if any(b"source" in s for s in parameters):
        condition = lambda i: True if b"source" in i else False
        links.append(["source_frame", [i.decode("utf-8") for i in parameters if condition(i)]])
    if any(b"phi" in s for s in parameters):
        condition = lambda i: True if b"phi" in i or b"tilt" in i else False
        links.append(["spin_angles", [i.decode("utf-8") for i in parameters if condition(i)]])
    if any(b"ra" in s for s in parameters):
        condition = lambda i: True if b"ra" == i or b"dec" == i or b"psi" == i \
                              else False
        links.append(["sky_location", [i.decode("utf-8") for i in parameters if condition(i)]])
    if any(b"snr" in s for s in parameters):
        links.append(["SNR", [i.decode("utf-8") for i in parameters if b"snr" in i]])
    if any(b"distance" in s for s in parameters):
        condition = lambda i: True if b"distance" in i or b"time" in i or \
                              b"redshift" in i else False
        links.append(["Distance and Time", [i.decode("utf-8") for i in parameters if condition(i)]])
    if any(b"phase" in s for s in parameters):
        condition = lambda i: True if b"phase" in i or b"likelihood" in i \
                              else False
        links.append(["Others", [i.decode("utf-8") for i in parameters if condition(i)]])
    return links

def make_home_pages(opts, approximants, samples, colors, parameters):
    """Make the home pages for all approximants

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    approximants: list
        list of approximants you wish to include
    samples: list
        list of samples you wish to include
    colors: list
        list of colors in hexadecimal format for the different approximants
    parameters: list
        list of parameters that the sampler varies over
    """
    pages = [i for i in approximants]
    pages.append("home")
    webpage.make_html(web_dir=opts.webdir, pages=pages)
    # design the home page
    html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                  html_page="home")
    html_file.make_header()
    if len(approximants) > 1:
        links = ["home", ["Approximants", [i for i in approximants]+["Comparison"]]]
    else:
        links = ["home", ["Approximants", [i for i in approximants]]]
    html_file.make_navbar(links=links)
    # make summary table of information
    likelihood = []
    subset = []
    for num, i in enumerate(samples):
        with h5py.File(i) as f:
            params = [j for j in f["parameter_names"]]
            index = params.index(b"log_likelihood")
            subset.append([j for j in f["samples"]])
            likelihood.append([j[index] for j in f["samples"]])
            f.close()
    same_parameters = list(set.intersection(*[set(l) for l in parameters]))
    data = []
    for i,j,k in zip(subset, likelihood, parameters):
        data.append(_grab_key_data(i,j,same_parameters,k))
        #data = [_grab_key_data(i, j, same_parameters, parameters) for i,j in zip(subset, likelihood)]
    contents = []
    for i in same_parameters:
        row = []
        row.append(i.decode("utf-8"))
        for j in range(len(samples)):
            row.append(np.round(data[j][i]["maxL"], 3))
        for j in range(len(samples)):
            row.append(np.round(data[j][i]["mean"], 3))
        for j in range(len(samples)):
            row.append(np.round(data[j][i]["median"], 3))
        for j in range(len(samples)):
            row.append(np.round(data[j][i]["std"], 3))
        contents.append(row)
    html_file.make_table(headings=[" ", "maxL", "mean", "median", "std"],
                         contents=contents, heading_span=len(samples),
                         colors=colors[:len(samples)])
    # design the home page for each approximant used
    for num, i in enumerate(approximants):
        html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                      html_page=i)
        html_file.make_header(title="{} Summary Page".format(i),
                              approximant="{}".format(i),
                              background_colour=colors[num])
        if len(approximants) > 1:
            links = ["home", ["Approximants", [k for k in approximants+["Comparison"]]],
                     "corner", "config", make_navbar_links(parameters[num])]
        else:
            links = ["home", ["Approximants", [k for k in approximants]],
                     "corner", "config", make_navbar_links(parameters[num])]
        html_file.make_navbar(links=links)
        # make an array of images that we want inserted in table
        contents = [["{}/plots/1d_posterior_{}_mass_1.png".format(opts.baseurl, i),
                     "{}/plots/1d_posterior_{}_mass_2.png".format(opts.baseurl, i),
                     "{}/plots/1d_posterior_{}_luminosity_distance.png".format(opts.baseurl, i)],
                    ["{}/plots/{}_skymap.png".format(opts.baseurl, i),
                     "{}/plots/{}_waveform.png".format(opts.baseurl, i)],
                    ["{}/plots/1d_posterior_{}_iota.png".format(opts.baseurl, i),
                     "{}/plots/1d_posterior_{}_a_1.png".format(opts.baseurl, i),
                     "{}/plots/1d_posterior_{}_a_2.png".format(opts.baseurl, i)]]
        html_file.make_table_of_images(contents=contents)
        images = [y for x in contents for y in x]
        html_file.make_modal_carousel(images=images)
        # make table of summary information
        contents = []
        for j in same_parameters:
            row = []
            row.append(j.decode("utf-8"))
            row.append(np.round(data[num][j]["maxL"], 3))
            row.append(np.round(data[num][j]["mean"], 3))
            row.append(np.round(data[num][j]["median"], 3))
            row.append(np.round(data[num][j]["std"], 3))
            contents.append(row)
        html_file.make_table(headings=[" ", "maxL", "mean", "median", "std"],
                             contents=contents, heading_span=1)
        html_file.make_footer(user=opts.user, rundir=opts.webdir)
        
def make_1d_histograms_pages(opts, approximants, samples, colors, parameters):
    """Make the 1d_histogram pages for all approximants

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    approximants: list
        list of approximants you wish to include
    samples: list
        list of samples you wish to include
    colors: list
        list of colors in hexadecimal format for the different approximants
    parameters: list
        list of parameters that the sampler varies over
    """
    for i in parameters:
        i.append(b"multiple")
    pages = ["{}_{}".format(i, j.decode("utf-8")) for num, i in enumerate(approximants) for j in parameters[num]]
    webpage.make_html(web_dir=opts.webdir, pages=pages)
    for app, col, par in zip(approximants, colors, parameters):
        if len(approximants) > 1:
            links = ["home", ["Approximants", [k for k in approximants+["Comparison"]]],
                     "corner", "config", make_navbar_links(par)]
        else:
            links = ["home", ["Approximants", [k for k in approximants]],
                     "corner", "config", make_navbar_links(par)]
        for j in par:
            html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                          html_page="{}_{}".format(app, j.decode("utf-8")))
            html_file.make_header(title="{} Posterior PDF for {}".format(app, j.decode("utf-8")),
                                  background_colour=col, approximant=app)
            html_file.make_navbar(links=links)

            if j != b"multiple":
                html_file.insert_image("{}/plots/1d_posterior_{}_{}.png".format(opts.baseurl,
                                                                                app,
                                                                                j.decode("utf-8")))
            else:
                html_file.make_search_bar(popular_options=["mass_1, mass_2",
                                                           "luminosity_distance, iota, ra, dec",
                                                           "iota, phi_12, phi_jl, tilt_1, tilt_2"],
                                          code="combines")
        html_file.make_footer(user=opts.user, rundir="{}".format(opts.webdir))
                

def make_comparison_pages(opts, approximants, samples, colors, parameters):
    """Make the comparison pages to compare all approximants

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    approximants: list
        list of approximants you wish to include
    samples: list
        list of samples you wish to include
    colors: list
        list of colors in hexadecimal format for the different approximants
    parameters: list
        list of parameters that the sampler varies over
    """
    webpage.make_html(web_dir=opts.webdir, pages=["Comparison"])
    html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                   html_page="Comparison")
    html_file.make_header(title="Comparison Summary Page")
    same_parameters = list(set.intersection(*[set(l) for l in parameters]))
    same_parameters.append(b"multiple")
    links = ["home", ["Approximant", [i for i in approximants]+["Comparison"]],
                     make_navbar_links(same_parameters)]
    html_file.make_navbar(links=links)
    contents = [["{}/plots/combined_skymap.png".format(opts.baseurl),
                 "{}/plots/compare_waveforms.png".format(opts.baseurl)]]
    html_file.make_table_of_images(contents=contents)
    if opts.sensitivity:
        html_file.add_content("<div class='row justify-content-center' "
                              "style='margin=top: 2.0em;'>"
                              "<p>To see the sky sensitivity for the following "
                              "networks, click the button</p></div>")
        html_file.add_content("<div class='row justify-content-center' "
                              "style='margin-top: 0.2em;'>"
                              "<button type='button' class='btn btn-info' "
                              "onclick='%s.src=\"%s/plots/combined_skymap.png\"'"
                              "style='margin-left:0.25em; margin-right:0.25em'>Sky Map</button>"
                              "<button type='button' class='btn btn-info' "
                              "onclick='%s.src=\"%s/plots/IMRPhenomPv2_sky_sensitivity_HL.png\"'"
                              "style='margin-left:0.25em; margin-right:0.25em'>HL</button>"
                               %("combined_skymap", opts.baseurl, "combined_skymap", opts.baseurl))
        html_file.add_content("<button type='button' class='btn btn-info' "
                              "onclick='%s.src=\"%s/plots/IMRPhenomPv2_sky_sensitivity_HLV.png\"'"
                              "style='margin-left:0.25em; margin-right:0.25em'>HLV</button></div>\n"
                               %("combined_skymap", opts.baseurl))

    html_file.make_footer(user=opts.user, rundir=opts.webdir)
    # edit all of the comparison pages
    pages = ["Comparison_{}".format(i.decode("utf-8")) for i in same_parameters]
    webpage.make_html(web_dir=opts.webdir, pages=pages)
    for i in same_parameters:
        html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                      html_page="Comparison_{}".format(i.decode("utf-8")))
        html_file.make_header(title="Comparison page for {}".format(i.decode("utf-8")),
                              approximant="Comparison")
        html_file.make_navbar(links=links)
        if i != b"multiple":
            html_file.insert_image("{}/plots/combined_posterior_{}.png".format(opts.baseurl,
                                                                               i.decode("utf-8")))
        else:
            html_file.make_search_bar(popular_options=["mass_1, mass_2",
                                                       "luminosity_distance, iota, ra, dec",
                                                       "iota, phi_12, phi_jl, tilt_1, tilt_2"],
                                      code="combines")
        html_file.make_footer(user=opts.user, rundir=opts.webdir)

def make_corner_pages(opts, approximants, samples, colors, parameters):
    """Make the corner pages for all approximants

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    approximants: list
        list of approximants you wish to include
    samples: list
        list of samples you wish to include
    colors: list
        list of colors in hexadecimal format for the different approximants
    parameters: list
        list of parameters that the sampler varies over
    """
    pages = ["{}_corner".format(i) for i in approximants]
    webpage.make_html(web_dir=opts.webdir, pages=pages)
    for app, col, par in zip(approximants, colors, parameters):
        if len(approximants) > 1:
            links = ["home", ["Approximants", [k for k in approximants+["Comparison"]]],
                     "corner", "config", make_navbar_links(par)]
        else:
            links = ["home", ["Approximants", [k for k in approximants]],
                     "corner", "config", make_navbar_links(par)]
        html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                      html_page="{}_corner".format(app))
        html_file.make_header(title="{} Corner plots".format(app), background_colour=col,
                              approximant=app)
        html_file.make_navbar(links=links)
        html_file.make_search_bar(popular_options=["mass_1, mass_2",
                                                       "luminosity_distance, iota, ra, dec",
                                                       "iota, phi_12, phi_jl, tilt_1, tilt_2"])
        html_file.make_footer(user=opts.user, rundir="{}".format(opts.webdir))

def make_config_pages(opts, approximants, samples, colors, configs, parameters):
    """Make the config pages for all approximants

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    approximants: list
        list of approximants you wish to include
    samples: list
        list of samples you wish to include
    colors: list
        list of colors in hexadecimal format for the different approximants
    configs: list
        list of paths to config files you wish to include
    parameters: list
        list of parameters that the sample varies over
    """
    pages = ["{}_config".format(i) for i in approximants]
    webpage.make_html(web_dir=opts.webdir, pages=pages, stylesheets=pages)
    if configs == None:
        configs = [None]*len(approximants)
    for app, con, col, par in zip(approximants, configs, colors, parameters):
        if len(approximants) > 1:
            links = ["home", ["Approximants", [k for k in approximants+["Comparison"]]],
                     "corner", "config", make_navbar_links(par)]
        else:
            links = ["home", ["Approximants", [k for k in approximants]],
                     "corner", "config", make_navbar_links(par)]
        html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                      html_page="{}_config".format(app))
        html_file.make_header(title="{} configuration".format(app), background_colour=col,
                              approximant=app)
        html_file.make_navbar(links=links)
        if con:
            with open(con, 'r') as f:
                contents = f.read()
            styles = html_file.make_code_block(language='ini', contents=contents)
            with open('{0:s}/css/{1:s}_config.css'.format(opts.webdir, app), 'w') as f:
                f.write(styles)
        else:
            html_file.add_content("<div class='row justify-content-center'>"
                                  "<p style='margin-top:2.5em'> No config file "
                                  "was provided </p></div>")
        html_file.make_footer(user=opts.user, rundir="{}".format(opts.webdir))

def write_html(opts, colors):
    """Generate an html page to show posterior plots

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    colors: list
        list of colors in hexadecimal format for the different approximants 
    """
    # grab the parameters
    parameters = [_grab_parameters(i) for i in opts.samples]
    # make the webpages
    make_home_pages(opts, opts.approximant, opts.samples, colors, parameters)
    make_1d_histograms_pages(opts, opts.approximant, opts.samples, colors,
                             parameters)
    make_corner_pages(opts, opts.approximant, opts.samples, colors, parameters)
    make_config_pages(opts, opts.approximant, opts.samples, colors, opts.config,
                      parameters)
    if len(opts.approximant) != 1:
        make_comparison_pages(opts, opts.approximant, opts.samples, colors,
                              parameters)

def write_html_data_dump(opts, colors):
    """Generate a single html page to show posterior plots
a
    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from command line
    colors: list
        list of colors in hexadecimal format for the different approximants
    """
    # grab the parameters from the samples
    f = h5py.File(opts.samples[0])
    parameters = [i for i in f["parameters"]]                       
    if b"log_likelihood" in parameters:                                          
        parameters.remove(b"log_likelihood")
    # make the relevant pages
    pages = ["home"]
    # links for all pages
    if len(opts.approximant) > 1:
        for i in opts.approximant:
            pages.append(i)
        links = ["home", ["Approximant", [i for i in opts.approximant]]]
    else:
        links = ["home"]
    # make the relevant pages
    webpage.make_html(web_dir=opts.webdir, pages=pages)
    if len(opts.approximant) > 1:
        # setup the home comparison page
        html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                      html_page="home")
        html_file.make_header()
        html_file.make_navbar(links=links)
        # content for accordian
        content = ["{}/plots/combined_posterior_{}.png".format(opts.baseurl, i) for i in parameters]
        html_file.make_accordian(headings=[i for i in parameters], content=content)
        html_file.make_footer(user=opts.user, rundir=opts.webdir)
    for num, i in enumerate(opts.approximant):
        if len(opts.approximant) == 1:
            html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                          html_page="home")
            html_file.make_header()
        else:
            html_file = webpage.open_html(web_dir=opts.webdir, base_url=opts.baseurl,
                                          html_page=i)
            html_file.make_header("{} Summary Page".format(i), background_colour=colors[num])
        html_file.make_navbar(links=links)
        # content for accordian
        content = ["{}/plots/1d_posterior_{}_{}.png".format(opts.baseurl, i, j) for j in parameters]
        html_file.make_accordian(headings=[i for i in parameters], content=content)
        html_file.make_footer(user=opts.user, rundir=opts.webdir)

if __name__ == '__main__':
    # default colors
    colors = ["#a6b3d0", "#baa997", "#FF6347", "#FFA500", "#003366"]
    # get arguments from command line
    parser = command_line()
    opts = parser.parse_args()
    # convert to the standard results file format
    logging.info("Converting files to standard format")
    convert_to_standard_format(opts.samples, opts.inj_file)
    # check the inputs
    logging.info("Checking the inputs")
    run_checks(opts)
    # make the plots for the webpage
    logging.info("Generating the plots")
    make_plots(opts, colors=colors)
    # copy over the relevant files to the web directory
    logging.info("Copying the files to %s" %(opts.webdir))
    copy_files(opts)
    # check to see if dump option is parsed
    logging.info("Writing HTML pages")
    if opts.dump:
        write_html_data_dump(opts, colors=colors)
    else:
        write_html(opts, colors=colors)
    # send email
    if opts.email:
        try:
            email_notify(opts.email, opts.baseurl+"/home.html")
        except Exception as e:
            print("Unable to send notification email because of error: {}".format(e))
    logging.info("Removing the temporary file that is in standard format")
    for i in opts.samples:
        os.remove(i)



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import pesummary
from pesummary.utils import utils
import sys
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import HtmlFormatter

def make_html(web_dir, title="Summary Pages", pages=None, stylesheets=[]):
    """Make the initial html page.

    Parameters
    ----------
    web_dir: str
        path to the location where you would like the html file to be saved
    title: str, optional
        header title of html page
    pages: list, optional
        list of pages that you would like to be created
    stylesheets: list, optional
        list of stylesheets to including in the html page. It is assumed all
        stylesheets are located in the `css` in the root of the summary pages.
        This should be provided as basenames without extension that is assumed
        to be `.css`.
    """
    for i in pages:
        if i != "home":
            i = "html/" + i
        f = open("{}/{}.html".format(web_dir, i), "w")
        doc_type = "<!DOCTYPE html>\n"
        stylesheet_elements = ''.join([
            "  <link rel='stylesheet' href='../css/{0:s}.css'>\n".format(s)
            for s in stylesheets])
        bootstrap = "<html lang='en'>\n" + \
                    "  <title>{}</title>\n".format(title) + \
                    "  <meta charset='utf-8'>\n" + \
                    "  <meta name='viewport' content='width=device-width, initial-scale=1'>\n" + \
                    "  <link rel='stylesheet' href='https://maxcdn.bootstrapcdn.com/bootstrap/4.1.3/css/bootstrap.min.css'>\n" + \
                    "  <script src='https://ajax.googleapis.com/ajax/libs/jquery/3.3.1/jquery.min.js'></script>\n" + \
                    "  <script src='https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.14.3/umd/popper.min.js'></script>\n" + \
                    "  <script src='https://maxcdn.bootstrapcdn.com/bootstrap/4.1.3/js/bootstrap.min.js'></script>\n" + \
                    stylesheet_elements + \
                    "</head>\n" + "<body style='background-color:#F8F8F8'>\n"
        f.writelines([doc_type, bootstrap])

def open_html(web_dir, base_url, html_page):
    """Open html page ready so you can manipulate the contents

    Parameters
    ----------
    web_dir: str
        path to the location where you would like the html file to be saved
    base_url: str
        url to the location where you would like the html file to be saved
    page: str
        name of the html page that you would like to edit
    """
    try:
        if html_page[-5:] == ".html":
            html_page = html_page[:-5]
    except:
        pass
    if html_page == "home.html" or html_page == "home":
        f = open(web_dir+"/home.html", "a")
    else:
        f = open(web_dir+"/html/"+html_page+".html", "a")
    return page(f, web_dir, base_url)

class page():
    """Class to generate and manipulate an html page.
    """
    def __init__(self, html_file, web_dir, base_url):
        self.html_file = html_file
        self.web_dir = web_dir
        self.base_url = base_url
        self.content = []

    def close(self):
        """Close the opened html file.
        """
        self.html_file.close()

    def add_content(self, content, indent=0):
        """Add content to the html page

        Parameters
        ----------
        content: str, optional
            string that you want to add to html page
        indent: int, optional
            the indent of the line
        """
        self.html_file.write(" "*indent + content)

    def _header(self, title, colour, approximant):
        """
        """
        self.add_content("<div class='jumbotron text-center' style='background-color: {}; margin-bottom:0'>\n".format(colour))
        self.add_content("  <h1 id={}>{}</h1>\n".format(approximant, title))
        self.add_content("<h4><span class='badge badge-info'>Code Version: %s"
                         "</span></h4>\n" %(pesummary.__version__), indent=2)
        self.add_content("</div>\n")

    def _footer(self, user, rundir):
        """
        """
        command= ""
        for i in sys.argv:
            command+=" {}".format(i)
        self.add_content("<div class='jumbotron text-center' style='margin-bottom:0'>\n")
        self.add_content("<p>Simulation run by {}. Run directories found at {}</p>\n".format(user, rundir), indent=2)
        self.add_content("<p>Command line: {}</p>\n".format(command), indent=2)
        self.add_content("</div>\n")

    def _setup_navbar(self):
        self.add_content("<script src='{}/js/variables.js'></script>\n".format(self.base_url))
        self.add_content("<script src='{}/js/grab.js'></script>\n".format(self.base_url))
        self.add_content("<script src='{}/js/multi_dropbar.js'></script>\n".format(self.base_url))
        self.add_content("<nav class='navbar navbar-expand-sm navbar-dark bg-dark'>\n")
        self.add_content("<a class='navbar-brand' href='#'>Navbar</a>\n", indent=2)
        self.add_content("<button class='navbar-toggler' type='button' "
                         "data-toggle='collapse' data-target='#collapsibleNavbar'>\n", indent=2)
        self.add_content("<span class='navbar-toggler-icon'></span>\n", indent=4)
        self.add_content("</button>\n", indent=2)

    def make_header(self, title="Parameter Estimation Summary Pages", background_colour="#eee",
                    approximant="IMRPhenomPv2"):
        """Make header for document in bootstrap format.

        Parameters
        ----------
        title: str, optional
            header title of html page
        approximant: str, optional
            the approximant that you are analysing
        """
        self._header(title, background_colour, approximant)

    def make_footer(self, user=None, rundir=None):
        """Make footer for document in bootstrap format.
        """
        self._footer(user, rundir)

    def make_navbar(self, links=None, search=True):
        """Make a navigation bar in boostrap format.

        Parameters
        ----------
        links: list, optional
            list giving links that you want your navbar to include. If a
            dropdown option is required, give a 2d list showing the main link
            followed by dropdown links. For instance, if you wanted to have
            links corner plots and a dropdown link named 1d_histograms with
            options mass1, mass2, mchirp, then we would give,

                links=[corner, [1d_histograms, [mass1, mass2, mchirp]]]
        search: bool, optional
            if True, search bar will be given in navbar
        """
        self._setup_navbar()
        if links == None:
            raise Exception ("Please specify links for use with navbar\n")
        self.add_content("<div class='collapse navbar-collapse' id='collapsibleNavbar'>\n", indent=4)
        self.add_content("<ul class='navbar-nav'>\n", indent=6)
        for i in links:
            if type(i) == list:
                self.add_content("<li class='nav-item'>\n", indent=8)
                self.add_content("<li class='nav-item dropdown'>\n", indent=10)
                self.add_content("<a class='nav-link dropdown-toggle' "
                                 "href='#' id='navbarDropdown' role='button' "
                                 "data-toggle='dropdown' aria-haspopup='true' "
                                 "aria-expanded='false'>\n", indent=12)
                self.add_content("{}\n".format(i[0]), indent=12)
                self.add_content("</a>\n", indent=12)
                self.add_content("<ul class='dropdown-menu' aria-labelledby='dropdown1'>\n", indent=12)
                for j in i:
                    if type(j) == list:
                        if len(j) > 1:
                            if type(j[1]) == list:
                                self.add_content("<li class='dropdown-item dropdown'>\n", indent=14)
                                self.add_content("<a class='dropdown-toggle' id='{}' "
                                                 "data-toggle='dropdown' "
                                                 "aria-haspopup='true' "
                                                 "aria-expanded='false'>{}</a>\n".format(j[0], j[0]), indent=16)
                                self.add_content("<ul class='dropdown-menu' "
                                                 "aria-labelledby='{}'>\n".format(j[0]), indent=16)
                                for k in j[1]:
                                    self.add_content("<li class='dropdown-item' href='#' "
                                                     "onclick='grab_html(\"{}\")'>"
                                                     "<a>{}</a></li>\n".format(k, k), indent=18)
                                self.add_content("</ul>", indent=16)
                                self.add_content("</li>", indent=14)
                            else:
                                for k in j:
                                    self.add_content("<li class='dropdown-item' href='#' "
                                                     "onclick='grab_html(\"{}\")'>"
                                                     "<a>{}</a></li>\n".format(k, k), indent=14)
                        else:
                            self.add_content("<li class='dropdown-item' href='#' "
                                             "onclick='grab_html(\"{}\")'>"
                                             "<a>{}</a></li>\n".format(j[0], j[0]), indent=14)
                self.add_content("</ul>\n", indent=12)
                self.add_content("</li>\n", indent=10)  
            else:
                self.add_content("<li class='nav-item'>\n", indent=8)
                if i == "home":
                    self.add_content("<a class='nav-link' "
                                     "href='{}/{}.html'>{}</a>\n".format(self.base_url, i, i), indent=10)
                else:
                    self.add_content("<a class='nav-link' "
                                     "href='#' onclick='grab_html(\"{}\")'"
                                     ">{}</a>\n".format(i, i), indent=10)
                self.add_content("</li>\n", indent=8)
        self.add_content("</ul>\n", indent=6)
        self.add_content("</div>\n", indent=4)

        if search:
            self.add_content("<input type='text' placeholder='search' id='search'>\n", indent=4)
            self.add_content("<script type='text/javascript' src='js/search.js'></script>\n", indent=4)
            self.add_content("<button type='submit' onclick='myFunction()'>Search</button>\n", indent=4)
        self.add_content("</nav>\n")

    def make_table(self, headings=None, contents=None, heading_span=1, colors=None):
        """Generate a table in bootstrap format.

        Parameters
        ----------
        headings: list, optional
            list of headings
        contents: list, optional
            nd list giving the contents of the table.
        heading_span: int, optional
            width of the header cell. By default it will span a single column
        colors: list, optional
            list of colors for the table columns
        """
        self.add_content("<div class='container' style='margin-top:5em'>\n")
        self.add_content("<div class='table-responsive'>\n", indent=2)
        if heading_span > 1:
            self.add_content("<table class='table table-sm'>\n", indent=4)
        else:
            self.add_content("<table class='table table-striped table-sm'>\n", indent=4)
        self.add_content("<thead>\n", indent=6)
        self.add_content("<tr>\n", indent=8)
        for i in headings:
            self.add_content("<th colspan='{}'>{}</th>\n".format(heading_span, i), indent=10)
        self.add_content("</tr>\n", indent=8)
        self.add_content("<tbody>\n", indent=6)

        for num, i in enumerate(contents):
            self.add_content("<tr>\n", indent=8)
            if heading_span == 2:
                for j, col in zip(i, ["#ffffff"]+colors*(len(i)-1)):
                    self.add_content("<td style='background-color: {}'>{}</td>\n".format(col, j), indent=10)
                self.add_content("</tr>", indent=8)
            else:
                for j in i:
                    self.add_content("<td>{}</td>\n".format(j), indent=10)
                self.add_content("</tr>\n", indent=8)

        self.add_content("</tbody>\n", indent=6)
        self.add_content("</table>\n", indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def make_code_block(self, language=None, contents=None):
        """Generate a code block hightlighted using pigments.

        Parameters
        ----------
        language: str, optional
            The lanaguge of the configuration file to use for syntax
            highlighting.
        contents: str, optional
            String containing the contents of the config file.

        Returns
        -------
        style: str
            The styles used to highlight the rendered contents.
        """
        lexer = get_lexer_by_name(language)
        formatter = HtmlFormatter(style='manni')
        render = highlight(contents, lexer, formatter)
        self.add_content(render)
        styles = formatter.get_style_defs('.highlight')
        styles += ".highlight {margin: 20px; padding: 20px;}"
        return styles

    def make_table_of_images(self, contents=None):
        """Generate a table of images in bootstrap format.

        Parameters
        ----------
        headings: list, optional
            list of headings
        contents: list, optional
            nd list giving the contents of the table.
        """
        self.add_content("<script type='text/javascript' src='../js/modal.js'></script>\n")
        self.add_content("<link rel='stylesheet' href='../css/image_styles.css'>\n")
        self.add_content("<div class='container' style='margin-top:5em; margin-bottom:5em;"
                         "background-color:#FFFFFF; box-shadow: 0 0 5px grey;'>\n")
        ind = 0
        for i in contents:
            self.add_content("<div class='row justify-content-center'>\n", indent=2)
            for num, j in enumerate(i):
                self.add_content("<div class='column'>\n", indent=4)
                self.add_content("<a href='#demo' data-slide-to='%s'>" %(ind), indent=6)
                self.add_content("<img src='{}'".format(self.base_url+"/plots/"+j.split("/")[-1]) +
                                 "alt='No image available' style='width:{}px;' "
                                 "id='{}' onclick='modal(\"{}\")'>\n".format(1050./len(i), j.split("/")[-1][:-4],
                                 j.split("/")[-1][:-4], indent=8))
                self.add_content("</a>", indent=6)
                self.add_content("</div>\n", indent=4)
                ind += 1
            self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def insert_image(self, path):
        """Generate an image in bootstrap format.

        Parameters
        ----------
        path: str, optional
            path to the image that you would like inserted
        """
        self.add_content("<div class='container' style='margin-top:5em; margin-bottom:5em;"
                         "background-color:#FFFFFF; box-shadow: 0 0 5px grey;'>\n") 
        self.add_content("<img src='{}' alt='No image available' "
                         "style='align-items:center; width:700px;'".format(path) +
                         "class='mx-auto d-block'>\n", indent=2)
        self.add_content("</p>\n", indent=2)
        self.add_content("<div style='clear: both;'></div>\n", indent=2)
        self.add_content("</div>\n")

    def make_accordian(self, headings=None, content=None):
        """Generate an accordian in bootstrap format with images as content.

        Parameters
        ----------
        headings: list, optional
            list of headings that you want your accordian to have
        content: nd list, optional
            n dimensional list where n is the number of rows. The content
            of each list should be the path to the location of the image
        """
        self.add_content("<div class='row justify-content-center'>\n")
        self.add_content("<div class='accordian' id='accordian' style='width:70%'>\n", indent=2)
        for num, i in enumerate(headings):
            self.add_content("<div class='card' style='border: 0px solid black'>\n", indent=4)
            self.add_content("<div class='card-header' id='{}'>\n".format(i), indent=6)
            self.add_content("<h5 class='mb-0'>\n", indent=8)
            self.add_content("<button class='btn btn-link collapsed' type='button' data-toggle='collapse' "
                             "data-target='#collapse{}' aria-expanded='false' ".format(i) +
                             "aria-controls='collapse{}'>\n".format(i), indent=10)
            self.add_content("{}\n".format(i), indent=12)
            self.add_content("</button>\n", indent=10)
            self.add_content("</h5>\n", indent=8)
            self.add_content("</div>\n", indent=6)
            self.add_content("<div id='collapse{}' class='collapse' ".format(i) +
                             "aria-labelledby='{}' data-parent='#accordian'>\n".format(i), indent=6)
            self.add_content("<div class='card-body'>\n", indent=8)
            self.add_content("<img src='{}' ".format(content[num]) +
                             "alt='No image available' style='width:700px;' " +
                             "class='mx-auto d-block'>\n", indent=10)
            self.add_content("</div>\n", indent=8)
            self.add_content("</div>\n", indent=6)
            self.add_content("</div>\n", indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def make_search_bar(self, popular_options=None, code="combine"):
        """Generate a search bar to combine the corner plots
        javascript.

        Parameters
        ----------
        popular_options: list, optional
            a list of popular options for your search bar
        """
        ids = "canvas" if code == "combine" else code
        self.add_content("<link rel='stylesheet' href='../css/side_bar.css'>\n")
        self.add_content("<script type='text/javascript' src='../js/combine_corner.js'></script>\n")
        self.add_content("<script type='text/javascript' src='../js/side_bar.js'></script>\n")
        self.add_content("<script type='text/javascript' src='../js/multiple_posteriors.js'></script>\n")
        self.add_content("<div class='w3-sidebar w3-bar-block w3-border-right sidenav' "
                         "style='display:none' id='mySidebar'>\n")
        self.add_content("<button onclick='side_bar_close()' class='close'>&times;</button>\n", indent=2)
        corner_parameters = ["luminosity_distance", "dec", "a_2",
                             "a_1", "geocent_time", "phi_jl", "psi", "ra", "phase",
                             "mass_2", "mass_1", "phi_12", "tilt_2", "iota",
                             "tilt_1", "chi_p", "chirp_mass", "mass_ratio",
                             "symmetric_mass_ratio", "total_mass", "chi_eff"]
        for i in corner_parameters:
            self.add_content("<input type='checkbox' name='type' "
                             "value='{}' id='{}' style='text-align: center; margin: 0 5px 0;'"
                             ">{}<br>\n".format(i, i,i,i), indent=2)
        self.add_content("</div>")
        self.add_content("<div class='row justify-content-center'>")
        self.add_content("<p style='margin-top:2.5em'> Input the parameter names that you would like to compare</p>", indent=2)
        self.add_content("</div>")
        self.add_content("<div class='row justify-content-center'>\n")
        self.add_content("<input type='text' placeholder='search' id='corner_search'>\n", indent=2)
        self.add_content("<button type='submit' onclick='{}()'>Submit</button>\n".format(code), indent=2)
        self.add_content("<button class='w3-button w3-teal w3-xlarge' "
                         "onclick='side_bar_open()'>&times </button>\n", indent=2) 
        self.add_content("</div>\n")
        self.add_content("<div class='row justify-content-center'>\n")
        if popular_options:
            for i in popular_options:
                self.add_content("<button type='button' class='btn btn-info' "
                                 "onclick='{}(\"{}\")' "
                                 "style='margin-left:0.25em; margin-right:0.25em; "
                                 "margin-top: 1.0em'>{}</button>\n".format(code, i, i), indent=2)
        self.add_content("</div>")
        self.add_content("<div class='container' style='margin-top:5em; margin-bottom:5em;"
                         "background-color:#FFFFFF; box-shadow: 0 0 5px grey;'>\n")  
        self.add_content("<div class='row justify-content-center' id='corner_plot'>\n", indent=2)
        self.add_content("<canvas id='{}' width='600' height='600'></canvas>\n".format(ids), indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")

    def make_modal_carousel(self, images=None):
        """Make a pop up window that appears on top of the home page showing
        images in a carousel.

        Parameters
        ----------
        images: list
            list of image locations that you would like included in the
            carousel
        """
        self.add_content("<div class='modal fade bs-example-modal-lg' tabindex='-1' "
                         "role='dialog' aria-labelledby='myLargeModalLabel' "
                         "aria-hidden='true' id='myModel' style='margin-top: 200px;'>\n")
        self.add_content("<div class='modal-dialog modal-lg' style='width:90%'>\n", indent=2)
        self.add_content("<div class='modal-content'>\n", indent=4)
        self.add_content("<div id='demo' class='carousel slide' data-ride='carousel'>\n", indent=6)
        self.add_content("<ul class='carousel-indicators'>\n", indent=8)
        for num, i in enumerate(images):
            if num == 0:
                self.add_content("<li data-target='#demo' data-slide-to-'%s' "
                                 "class='active'></li>\n" %(num), indent=10)
            self.add_content("<li data-target='#demo' data-slide-to-'%s'>"
                             "</li>\n" %(num), indent=10)
        self.add_content("<li data-target='#demo' data-slide-to='0' "
                         "class='active'></li>\n", indent=10)
        self.add_content("<li data-target='#demo' data-slide-to='1'></li>\n", indent=10)
        self.add_content("<li data-target='#demo' data-slide-to='2'></li>\n", indent=10)
        self.add_content("</ul>\n", indent=8)
        self.add_content("<div class='carousel-inner'>\n", indent=8)
        for num, i in enumerate(images):
            if num == 0:
                self.add_content("<div class='carousel-item active'>\n", indent=10)
            else:
                self.add_content("<div class='carousel-item'>\n", indent=10)
            self.add_content("<img src={} style='align-items:center;' "
                             "class='mx-auto d-block'>\n".format(i), indent=12)
            self.add_content("</div>\n", indent=10)

        self.add_content("</div>\n", indent=8)
        self.add_content("<a class='carousel-control-prev' href='#demo' "
                         "data-slide='prev'>\n", indent=8)
        self.add_content("<span class='carousel-control-prev-icon'></span>\n", indent=10)
        self.add_content("</a>\n", indent=8)
        self.add_content("<a class='carousel-control-next' href='#demo' "
                         "data-slide='next'>\n", indent=8)
        self.add_content("<span class='carousel-control-next-icon'></span>\n", indent=10)
        self.add_content("</a>\n", indent=8)
        self.add_content("</div>\n", indent=6)
        self.add_content("</div>\n", indent=4)
        self.add_content("</div>\n", indent=2)
        self.add_content("</div>\n")



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import logging
logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)

import h5py
import deepdish

import numpy as np

try:
    from glue.ligolw import ligolw
    from glue.ligolw import lsctables
    from glue.ligolw import utils as ligolw_utils
    GLUE=True
except:
    GLUE=False

try:
    from lalsimulation import SimInspiralTransformPrecessingNewInitialConditions
    from lal import MSUN_SI
    LALINFERENCE_INSTALL=True
except:
    LALINFERENCE_INSTALL=False

try:
    from astropy.cosmology import z_at_value, Planck15
    import astropy.units as u
    ASTROPY=True
except ImportError:
    ASTROPY=False
    logger.warning("You do not have astropy installed currently. You will"
                   " not be able to use some of the prebuilt functions.")


def _make_hdf5_file(name, data, parameters, approximant, inj_par=None,
                    inj_data=None):
    """
    """
    f = h5py.File("%s_temp" %(name), "w")
    f.create_dataset("parameter_names", data=parameters)
    f.create_dataset("samples", data=data)
    f.create_dataset("approximant", data=approximant)
    f.create_dataset("injection_parameters", data=inj_par)
    f.create_dataset("injection_data", data=inj_data)
    f.close()

@np.vectorize
def _z_from_dL(luminosity_distance):
    """Return the redshift given samples for the luminosity distance
    """
    return z_at_value(Planck15.luminosity_distance, luminosity_distance*u.Mpc)

def _dL_from_z(redshift):
    """Return the luminosity distance given samples for the redshift
    """
    return Planck15.luminosity_distance(redshift).value

def _comoving_distance_from_z(redshift):
    """Return the comoving distance given samples for the redshift
    """
    return Planck15.comoving_distance(redshift).value 

def _m1_source_from_m1_z(mass1, z):
    """Return the source mass of the bigger black hole given samples for the
    detector mass of the bigger black hole and the redshift
    """
    return mass1 / (1. + z)

def _m2_source_from_m2_z(mass2, z):
    """Return the source mass of the smaller black hole given samples for the
    detector mass of the smaller black hole and the redshift
    """
    return mass2 / (1. + z)

def _m_total_source_from_mtotal_z(total_mass, z):
    """Return the source total mass of the binary given samples for detector
    total mass and redshift
    """
    return total_mass / (1. + z)

def _mchirp_source_from_mchirp_z(mchirp, z):
    """Return the source chirp mass of the binary given samples for detector
    chirp mass and redshift
    """
    return mchirp / (1. + z)

def _mchirp_from_m1_m2(mass1, mass2):
    """Return the chirp mass given the samples for mass1 and mass2

    Parameters
    ----------
    """
    return (mass1 * mass2)**0.6 / (mass1 + mass2)**0.2

def _m_total_from_m1_m2(mass1, mass2):
    """Return the total mass given the samples for mass1 and mass2
    """
    return mass1 + mass2

def _m1_from_mchirp_q(mchirp, q):
    """Return the mass of the larger black hole given the chirp mass and
    mass ratio
    """
    return (q**(2./5.))*((1.0 + q)**(1./5.))*mchirp 

def _m2_from_mchirp_q(mchirp, q):
    """Return the mass of the smaller black hole given the chirp mass and
    mass ratio
    """
    return (q**(-3./5.))*((1.0 + q)**(1./5.))*mchirp

def _eta_from_m1_m2(mass1, mass2):
    """Return the symmetric mass ratio given the samples for mass1 and mass2
    """
    return (mass1 * mass2) / (mass1 + mass2)**2

def _q_from_m1_m2(mass1, mass2):
    """Return the mass ratio given the samples for mass1 and mass2
    """
    return mass1 / mass2

def _q_from_eta(symmetric_mass_ratio):
    """Return the mass ratio given samples for symmetric mass ratio
    """
    temp = (1 / symmetric_mass_ratio / 2 - 1)
    return temp - (temp ** 2 - 1) ** 0.5

def _mchirp_from_mtotal_q(total_mass, mass_ratio):
    """Return the chirp mass given samples for total mass and mass ratio
    """
    mass1 = mass_ratio*total_mass / (1.+mass_ratio)
    mass2 = total_mass / (1.+mass_ratio)
    return _eta_from_m1_m2(mass1, mass2)**(3./5) * (mass1+mass2)

def _chi_p(mass1, mass2, spin1x, spin1y, spin2x, spin2y):
    """Return chi_p given samples for mass1, mass2, spin1x, spin1y, spin2x,
    spin2y
    """
    mass_ratio = mass1/mass2
    B1 = 2.0 + 1.5*mass_ratio
    B2 = 2.0 + 3.0 / (2*mass_ratio)
    S1_perp = ((spin1x)**2 + (spin1y)**2)**0.5
    S2_perp = ((spin2x)**2 + (spin2y)**2)**0.5
    chi_p = 1.0/B1 * np.maximum(B1*S1_perp, B2*S2_perp)
    return chi_p

def _chi_eff(mass1, mass2, spin1z, spin2z):
    """Return chi_eff given samples for mass1, mass2, spin1z, spin2z
    """
    return (spin1z * mass1 + spin2z * mass2) / (mass1 + mass2)

def _component_spins(theta_jn, phi_jl, tilt_1, tilt_2, phi_12, a_1, a_2, mass_1,
                     mass_2, f_ref, phase):
    """Return the component spins given sames for theta_jn, phi_jl, tilt_1,
    tilt_2, phi_12, a_1, a_2, mass_1, mass_2, f_ref, phase
    """
    if LALINFERENCE_INSTALL:
        data = []
        for i in range(len(theta_jn)):
            iota, S1x, S1y, S1z, S2x, S2y, S2z = \
                SimInspiralTransformPrecessingNewInitialConditions(
                    theta_jn[i], phi_jl[i], tilt_1[i], tilt_2[i], phi_12[i],
                    a_1[i], a_2[i], mass_1[i]*MSUN_SI, mass_2[i]*MSUN_SI,
                    f_ref[i], phase[i])
            data.append([iota, S1x, S1y, S1z, S2x, S2y, S2z])
        return data
    else:
        raise Exception("Please install LALSuite for full conversions")

def all_parameters(data, parameters):
    """Return an array of samples for all CBC parameters

    Parameters
    ----------
    data: list
        list of samples returned from your sampler of choice
    parameters: list
        list of parameters that have been sampled over
    """
    if "mass_ratio" not in parameters and "symmetric_mass_ratio" in parameters:
        parameters.append("mass_ratio")
        symmetric_mass_ratio_ind = parameters.index("symmetric_mass_ratio")
        symmetric_mass_ratio = np.array([i[symmetric_mass_ratio_ind] for i in data])
        mass_ratio = _q_from_eta(symmetric_mass_ratio)
        for num, i in enumerate(data):
            data[num].append(mass_ratio[num])
    if "mass_ratio" not in parameters and "mass_1" in parameters and "mass_2" in parameters:
        parameters.append("mass_ratio")
        mass1_ind = parameters.index("mass_1")
        mass1 = np.array([i[mass1_ind] for i in data])
        mass2_ind = parameters.index("mass_2")
        mass2 = np.array([i[mass2_ind] for i in data])
        q = _q_from_m1_m2(mass1, mass2)
        for num, i in enumerate(data):
            data[num].append(q[num])
    if "mass_ratio" in parameters:
        mass_ratio_ind = parameters.index("mass_ratio")
        mass_ratio = np.array([i[mass_ratio_ind] for i in data])
        median = np.median(mass_ratio)
        if median < 1:
            # define mass_ratio so that q>1
             for i in data:
                 i[mass_ratio_ind] = 1/i[mass_ratio_ind]
    if "chirp_mass" not in parameters and "total_mass" in parameters:
        parameters.append("chirp_mass")
        total_mass_ind = parameters.index("total_mass")
        mass_ratio_ind = parameters.index("mass_ratio")
        total_mass = np.array([i[total_mass_ind] for i in data])
        mass_ratio = np.array([i[mass_ratio_ind] for i in data])
        chirp_mass = _mchirp_from_mtotal_q(total_mass, mass_ratio)
        for num, i in enumerate(data):
            data[num].append(chirp_mass[num])
    if "mass_1" not in parameters and "chirp_mass" in parameters:
        parameters.append("mass_1")
        chirp_mass_ind = parameters.index("chirp_mass")
        mass_ratio_ind = parameters.index("mass_ratio")
        chirp_mass = np.array([i[chirp_mass_ind] for i in data])
        mass_ratio = np.array([i[mass_ratio_ind] for i in data])
        mass_1 = _m1_from_mchirp_q(chirp_mass, mass_ratio)
        for num, i in enumerate(data):
            data[num].append(mass_1[num])
    if "mass_2" not in parameters and "chirp_mass" in parameters:
        parameters.append("mass_2")
        chirp_mass_ind = parameters.index("chirp_mass")
        mass_ratio_ind = parameters.index("mass_ratio")
        chirp_mass = np.array([i[chirp_mass_ind] for i in data])
        mass_ratio = np.array([i[mass_ratio_ind] for i in data])
        mass_2 = _m2_from_mchirp_q(chirp_mass, mass_ratio)
        for num, i in enumerate(data):
            data[num].append(mass_2[num])

    ##################################################
    # True reference frequency needs to be extracted #
    #              NEEDS TO BE FIXED                 #
    ##################################################

    if "reference_frequency" not in parameters:
        parameters.append("reference_frequency")
        for num, i in enumerate(data):
            data[num].append(20.)

    if "mass_1" in parameters and "mass_2" in parameters:
        mass1_ind = parameters.index("mass_1")
        mass1 = np.array([i[mass1_ind] for i in data])
        mass2_ind = parameters.index("mass_2")
        mass2 = np.array([i[mass2_ind] for i in data])
        if "total_mass" not in parameters:
            parameters.append("total_mass")
            m_total = _m_total_from_m1_m2(mass1, mass2)
            for num, i in enumerate(data):
                data[num].append(m_total[num])
        if "chirp_mass" not in parameters:
            parameters.append("chirp_mass")
            mchirp = _mchirp_from_m1_m2(mass1, mass2)
            for num, i in enumerate(data):
                data[num].append(mchirp[num])
        if "symmetric_mass_ratio" not in parameters:
            parameters.append("symmetric_mass_ratio")
            eta = _eta_from_m1_m2(mass1, mass2)
            for num, i in enumerate(data):
                data[num].append(eta[num])

        spin_components = ["spin_1x", "spin_1y", "spin_1z", "spin_2x",
                           "spin_2y", "spin_2z"]
        spin_angles = ["iota", "phi_jl", "tilt_1", "tilt_2", "phi_12", "a_1", "a_2",
                       "mass_1", "mass_2", "reference_frequency", "phase"]
        if all(i not in parameters for i in spin_components):
            if all(i in parameters for i in spin_angles):
                parameters.append("spin_1x")
                parameters.append("spin_1y")
                parameters.append("spin_1z")
                parameters.append("spin_2x")
                parameters.append("spin_2y")
                parameters.append("spin_2z")
                indices = [parameters.index("%s" %(i)) for i in spin_angles]
                iota = np.array([i[indices[0]] for i in data])
                phijl = np.array([i[indices[1]] for i in data])
                tilt1 = np.array([i[indices[2]] for i in data])
                tilt2 = np.array([i[indices[3]] for i in data])
                phi12 = np.array([i[indices[4]] for i in data])
                a1 = np.array([i[indices[5]] for i in data])
                a2 = np.array([i[indices[6]] for i in data])
                mass1 = np.array([i[indices[7]] for i in data])
                mass2 = np.array([i[indices[8]] for i in data])
                f_ref = np.array([i[indices[9]] for i in data])
                phase = np.array([i[indices[10]] for i in data])
                spin_parameters = _component_spins(iota, phijl, tilt1, tilt2, phi12,
                                                   a1, a2, mass1, mass2, f_ref, phase)
                for num, i in enumerate(data):
                    data[num].append(spin_parameters[num][1])
                    data[num].append(spin_parameters[num][2])
                    data[num].append(spin_parameters[num][3])
                    data[num].append(spin_parameters[num][4])
                    data[num].append(spin_parameters[num][5])
                    data[num].append(spin_parameters[num][6])
        if "chi_p" not in parameters and "chi_eff" not in parameters:
            if all(i in parameters for i in spin_angles):
                parameters.append("chi_p")
                parameters.append("chi_eff")
                indices = [parameters.index("%s" %(i)) for i in spin_components]
                spin1x = np.array([i[indices[0]] for i in data])
                spin1y = np.array([i[indices[1]] for i in data])
                spin1z = np.array([i[indices[2]] for i in data])
                spin2x = np.array([i[indices[3]] for i in data])
                spin2y = np.array([i[indices[4]] for i in data])
                spin2z = np.array([i[indices[5]] for i in data])
                chi_p = _chi_p(mass1, mass2, spin1x, spin1y, spin2x, spin2y)
                chi_eff = _chi_eff(mass1, mass2, spin1z, spin2z)
                for num, i in enumerate(data):
                    data[num].append(chi_p[num])
                    data[num].append(chi_eff[num])
    if "cos_tilt_1" not in parameters and "tilt_1" in parameters:
        parameters.append("cos_tilt_1")
        tilt_1_ind = parameters.index("tilt_1")
        tilt_1 = np.array([i[tilt_1_ind] for i in data])
        cos_tilt_1 = np.cos(tilt_1)
        for num, i in enumerate(data):
            data[num].append(cos_tilt_1[num])
    if "cos_tilt_2" not in parameters and "tilt_2" in parameters:
        parameters.append("cos_tilt_2")
        tilt_2_ind = parameters.index("tilt_2")
        tilt_2 = np.array([i[tilt_2_ind] for i in data])
        cos_tilt_2 = np.cos(tilt_2)
        for num, i in enumerate(data):
            data[num].append(cos_tilt_2[num])
    if "luminosity_distance" not in parameters and "redshift" in parameters:
        parameters.append("luminosity_distance")
        redshift_ind = parameters.index("redshift")
        redshift = np.array([i[redshift_ind] for i in data])
        luminosity_distance = _dL_from_z(redshift)
        for num, i in enumerate(data):
            data[num].append(luminosity_distance[num])
    if "redshift" not in parameters and "luminosity_distance" in parameters:
        parameters.append("redshift")
        luminosity_distance_ind = parameters.index("luminosity_distance")
        luminosity_distance = np.array([i[luminosity_distance_ind] for i in data])
        redshift = _z_from_dL(luminosity_distance)
        for num, i in enumerate(data):
            data[num].append(redshift[num])
    if "comoving_distance" not in parameters and "redshift" in parameters:
        parameters.append("comoving_distance")
        redshift_ind = parameters.index("redshift")
        redshift = np.array([i[redshift_ind] for i in data])
        comoving_distance = _comoving_distance_from_z(redshift)
        for num, i in enumerate(data):
            data[num].append(comoving_distance[num])
    if "redshift" in parameters:
        redshift_ind = parameters.index("redshift")
        redshift = np.array([i[redshift_ind] for i in data])
        if "mass_1_source" not in parameters and "mass_1" in parameters:
            parameters.append("mass_1_source")
            mass_1_ind = parameters.index("mass_1")
            mass_1 = np.array([i[mass_1_ind] for i in data])
            mass_1_source = _m1_source_from_m1_z(mass_1, redshift)
            for num, i in enumerate(data):
                data[num].append(mass_1_source[num])
        if "mass_2_source" not in parameters and "mass_2" in parameters:
            parameters.append("mass_2_source")
            mass_2_ind = parameters.index("mass_2")
            mass_2 = np.array([i[mass_2_ind] for i in data])
            mass_2_source = _m2_source_from_m2_z(mass_2, redshift)
            for num, i in enumerate(data):
                data[num].append(mass_2_source[num])
        if "total_mass_source" not in parameters and "total_mass" in parameters:
            parameters.append("total_mass_source")
            total_mass_ind = parameters.index("total_mass")
            total_mass = np.array([i[total_mass_ind] for i in data])
            total_mass_source = _m_total_source_from_mtotal_z(total_mass, redshift)
            for num, i in enumerate(data):
                data[num].append(total_mass_source[num])
        if "chirp_mass_source" not in parameters and "chirp_mass" in parameters:
            parameters.append("chirp_mass_source")
            chirp_mass_ind = parameters.index("chirp_mass")
            chirp_mass = np.array([i[chirp_mass_ind] for i in data])
            chirp_mass_source = _mchirp_source_from_mchirp_z(chirp_mass, redshift)
            for num, i in enumerate(data):
                data[num].append(chirp_mass_source[num])
    return data, parameters

def one_format(fil, inj):
    """Looks at the input file and puts it into a standard form such that all
    parameter estimation codes can use the summary pages.

    Parameters
    ----------
    fil: str
        path to the results file location
    inj: str
        path to the location of the injection file
    """
    LALINFERENCE = False
    BILBY = False
    file_location = "/".join(fil.split("/")[:-1])
    f = h5py.File(fil)
    keys = [i for i in f.keys()]
    if "lalinference" in keys:
        LALINFERENCE = True
        logging.info("LALInference was used to generate %s" %(fil))
    elif "data" in keys:
        BILBY = True
        logging.info("BILBY >= v0.3.3 was used to generate %s" %(fil))
        path = "data/posterior"
    elif "posterior" in keys:
        BILBY = True
        logging.info("BILBY >= v0.3.1 <= v0.3.3 was used to generate %s" %(fil))
        path = "posterior"
    else:
        raise Exception("Data format not understood")
    if LALINFERENCE:
        standard_names = {"logL": "log_likelihood",
                          "logl": "log_likelihood",
                          "tilt1": "tilt_1",
                          "tilt_spin1": "tilt_1",
                          "tilt2": "tilt_2",
                          "tilt_spin2": "tilt_2",
                          "costilt1": "cos_tilt_1",
                          "costilt2": "cos_tilt_2",
                          "redshift": "redshift",
                          "phi_jl": "phi_jl",
                          "l1_optimal_snr": "L1_optimal_snr",
                          "h1_optimal_snr": "H1_optimal_snr",
                          "v1_optimal_snr": "V1_optimal_snr",
                          "L1_optimal_snr": "L1_optimal_snr",
                          "H1_optimal_snr": "H1_optimal_snr",
                          "V1_optimal_snr": "V1_optimal_snr",
                          "E1_optimal_snr": "E1_optimal_snr",
                          "mc_source": "chirp_mass_source",
                          "eta": "symmetric_mass_ratio",
                          "m1": "mass_1",
                          "m2": "mass_2",
                          "ra": "ra",
                          "rightascension": "ra",
                          "dec": "dec",
                          "declination": "dec",
                          "iota": "iota",
                          "m2_source": "mass_2_source",
                          "m1_source": "mass_1_source",
                          "phi1": "phi_1",
                          "phi2": "phi_2",
                          "psi": "psi",
                          "polarisation": "psi",
                          "phi12": "phi_12",
                          "a1": "a_1",
                          "a_spin1": "a_1",
                          "a2": "a_2",
                          "a_spin2": "a_2",
                          "chi_p": "chi_p",
                          "phase": "phase",
                          "distance": "luminosity_distance",
                          "dist": "luminosity_distance",
                          "mc": "chirp_mass",
                          "chirpmass": "chirp_mass",
                          "chi_eff": "chi_eff",
                          "mtotal_source": "total_mass_source",
                          "mtotal": "total_mass",
                          "q": "mass_ratio",
                          "time": "geocent_time",
                          "theta_jn": "iota"}
        approx = "none"
        sampler = [i for i in f["lalinference"].keys()]
        data_path = "lalinference/%s/posterior_samples" %(sampler[0])
        lalinference_names = f[data_path].dtype.names
        parameters, data = [], []
        for i in lalinference_names:
            if i in standard_names.keys():
                parameters.append(i)
        for i in f[data_path]:
            data.append([i[lalinference_names.index(j)] for j in parameters])
        parameters = [standard_names[i] for i in parameters]
        index = lalinference_names.index
        if "luminosity_distance" not in parameters and "logdistance" in lalinference_names:
            parameters.append("luminosity_distance")
            for num, i in enumerate(f[data_path]):
                data[num].append(np.exp(i[index("logdistance")]))
        if "iota" not in parameters and "costheta_jn" in lalinference_names:
            parameters.append("iota")
            for num, i in enumerate(f[data_path]):
                data[num].append(np.arccos(i[index("costheta_jn")]))
    if BILBY:
        approx = "none"
        inj_par, inj_data = [], []
        parameters, data = [], []
        try:
            logging.info("Trying to load with file with deepdish")
            f = deepdish.io.load(fil)
            parameters, data, approx = load_with_deepdish(f)
        except:
            logging.info("Failed to load file with deepdish. Using h5py to "
                         "load in data")
            f = h5py.File(fil)
            parameters, data = load_with_h5py(f, path)
    data, parameters = all_parameters(data, parameters) 
    if "reference_frequency" in parameters:
        index = parameters.index("reference_frequency")
        parameters.remove(parameters[index])
        for i in data:
            i.remove(i[index])
    if "minimum_frequency" in parameters:
        index = parameters.index("minimum_frequency")
        parameters.remove(parameters[index])
        for i in data:
            i.remove(i[index])
    if "logPrior" in parameters:
        index = parameters.index("logPrior")
        parameters.remove(parameters[index])
        for i in data:
            i.remove(i[index])
    if LALINFERENCE:
        inj_par, inj_data = get_injection_parameters(parameters, inj, 
                                                     LALINFERENCE=True)
    if BILBY:
        inj_par, inj_data = get_injection_parameters(parameters, fil, BILBY=True)
    _make_hdf5_file(fil, np.array(data), np.array(parameters, dtype="S"),
                    np.array([approx], dtype="S"),
                    inj_par = np.array(inj_par, dtype="S"),
                    inj_data = np.array(inj_data))
    return "%s_temp" %(fil)

def load_with_deepdish(f):
    """Return the data and parameters that appear in a given h5 file assuming
    that the file has been loaded with deepdish

    Parameters
    ----------
    f: dict
        results file loaded with deepdish 
    """
    approx = "none"
    parameters = [i for i in f["posterior"].keys()]
    if "waveform_approximant" in parameters:
        approx = f["posterior"]["waveform_approximant"][0]
        parameters.remove("waveform_approximant")
    data = np.zeros([len(f["posterior"]), len(parameters)])
    for num, par in enumerate(parameters):
        for key, i in enumerate(f["posterior"][par]):
            data[key][num] = float(np.real(i))
    data = data.tolist()
    for num, par in enumerate(parameters):
        if par == "logL":
            parameters[num] = "log_likelihood"
    return parameters, data, approx

def load_with_h5py(f, path):
    """Return the data and parameters that appear in a given h5 file assuming
    that the file has been loaded with h5py

    Parameters
    ----------
    f: h5py._hl.files.File
        results file loaded with h5py 
    """
    parameters, data = [], []
    blocks = [i for i in f["%s" %(path)] if "block" in i]
    for i in blocks:
        block_name = i.split("_")[0]
        if "items" in i:
            for par in f["%s/%s" %(path,i)]:
                if par == b"waveform_approximant":
                    blocks.remove(block_name+"_items")
                    blocks.remove(block_name+"_values")
    for i in sorted(blocks):
        if "items" in i:
            for par in f["%s/%s" %(path,i)]:
                if par == b"logL":
                    parameters.append(b"log_likelihood")
                else:
                    parameters.append(par)
        if "values" in i:
            if len(data) == 0:
                for dat in f["%s/%s" %(path, i)]:
                    data.append(list(np.real(dat)))
            else:
                for num, dat in enumerate(f["%s/%s" %(path, i)]):
                    data[num] += list(np.real(dat))
    parameters = [i.decode("utf-8") for i in parameters]
    return parameters, data

def get_injection_parameters(parameters, inj_file, LALINFERENCE=False,
                             BILBY=False):
    """Grab the injection parameters from an xml injection file

    Parameters
    ----------
    parameters: list
        list of parameters that you have samples for
    inj_file: str
        path to the location of the injection file
    """
    _q_func = _q_from_m1_m2
    _eta_func = _eta_from_m1_m2
    _M_func = _m_total_from_m1_m2
    func_map = {"chirp_mass": lambda inj:inj.mchirp,
                "symmetric_mass_ratio": lambda inj: inj.eta,
                "luminosity_distance": lambda inj: inj.distance,
                "mass_1": lambda inj: inj.mass1,
                "mass_2": lambda inj: inj.mass2,
                "dec": lambda inj: inj.latitude,
                "spin_1x": lambda inj: inj.spin1x,
                "spin_1y": lambda inj: inj.spin1y,
                "spin_1z": lambda inj: inj.spin1z,
                "spin_2x": lambda inj: inj.spin2x,
                "spin_2y": lambda inj: inj.spin2y,
                "spin_2z": lambda inj: inj.spin2z,
                "mass_ratio": lambda inj: _q_func(inj.mass1, inj.mass2),
                "symmetric_mass_ratio": lambda inj: _eta_func(inj.mass1, inj.mass2),
                "total_mass": lambda inj: inj.mass1+inj.mass2,
                "chi_p": lambda inj: _chi_p(inj.mass1, inj.mass2, inj.spin1x,
                                            inj.spin1y, inj.spin2x, inj.spin2y),
                "chi_eff": lambda inj: _chi_eff(inj.mass1, inj.mass2, inj.spin1z,
                                                inj.spin2z)}
    inj_par = parameters
    if LALINFERENCE:
        if inj_file == None:
            inj_data = [float("nan")]*len(parameters)
        else:
            if GLUE:
                xmldoc = ligolw_utils.load_filename(inj_file, contenthandler= \
                                     lsctables.use_in(ligolw.LIGOLWContentHandler))
                table=lsctables.SimInspiralTable.get_table(xmldoc)[0]
                inj_data = [func_map[i](table) if i in func_map.keys() else \
                            float("nan") for i in parameters]
            else:
                inj_data = [float("nan")]*len(parameters)
    if BILBY:
        try:
            f = deepdish.io.load(inj_file)
            inj_keys = f["injection_parameters"].keys()
            inj_data = [f["injection_parameters"][key] if key in inj_keys \
                          else float("nan") for key in parameters]
        except:
            inj_data = [float("nan")]*len(parameters)
    return inj_par, inj_data



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import logging
logging.basicConfig(format='%(asctime)s %(message)s', level=logging.INFO)

import matplotlib
matplotlib.use("Agg")
import matplotlib.pyplot as plt
import corner

import numpy as np
from scipy.ndimage import gaussian_filter
from astropy.time import Time

from lal import MSUN_SI, PC_SI
try:
    import lalsimulation as lalsim
    LALSIMULATION=True
except:
    LALSIMULATION=None

def _1d_histogram_plot(param, samples, latex_label, inj_value=None):
    """Generate the 1d histogram plot for a given parameter for a given
    approximant.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    samples: list
        list of samples for param
    latex_label: str
        latex label for param
    inj_value: float
        value that was injected
    """
    logging.info("Generating the 1d histogram plot for %s" %(param.decode("utf-8")))
    fig = plt.figure()
    n, bins, patches = plt.hist(samples, histtype="step", bins=50, color='b')
    plt.xlabel(latex_label, fontsize=16)
    plt.ylabel("Probability Density", fontsize=16)
    upper_percentile = np.percentile(samples, 90)
    lower_percentile = np.percentile(samples, 10)
    y_range = [0,np.max(n)+0.1*np.max(n)]
    if inj_value:
        plt.plot([inj_value]*2, y_range, color='r', linestyle='--')
    plt.plot([upper_percentile]*2, y_range, color='b', linestyle='--')
    plt.plot([lower_percentile]*2, y_range, color='b', linestyle='--')
    median = np.median(samples)
    upper = np.round(upper_percentile - median, 2)
    lower = np.round(median - lower_percentile, 2)
    median = np.round(median, 2)
    plt.title(r"$%s^{+%s}_{-%s}$" %(median, upper, lower), fontsize=18)
    plt.grid()
    plt.ylim(y_range)
    return fig

def _1d_comparison_histogram_plot(param, approximants, samples, colors,
                                  latex_label):
    """Generate the a plot to compare the 1d_histogram plots for a given
    parameter for different approximants.

    Parameters
    ----------
    param: str
        name of the parameter that you wish to plot
    approximants: list
        list of approximant names that you would like to compare
    samples: 2d list
        list of samples for param for each approximant
    colors: list
        list of colors to be used to differentiate the different approximants
    latex_label: str
        latex label for param
    """
    logging.info("Generating the 1d comparison histogram plot for %s" %(param.decode("utf-8"))) 
    fig = plt.figure()
    for num, i in enumerate(samples):
        plt.hist(i, histtype="step", bins=50, color=colors[num],
                 label=approximants[num], linewidth=2.0)
        plt.axvline(x=np.percentile(i, 90), color=colors[num], linestyle='--',
                    linewidth=2.0)
        plt.axvline(x=np.percentile(i, 10), color=colors[num], linestyle='--',
                    linewidth=2.0)
    plt.xlabel(latex_label, fontsize=16)
    plt.ylabel("Probability Density", fontsize=16)
    plt.legend(loc="best")
    plt.grid()
    return fig

def __antenna_response(name, ra, dec, psi, time_gps):
    """Calculate the antenna response function

    Parameters
    ----------
    name: str
        name of the detector you wish to calculate the antenna response
        function for
    ra: float
        right ascension of the source
    dec: float
        declination of the source
    psi: float
        polarisation of the source
    time_gps: float
        gps time of merger
    """
    gmst = Time(time_gps, format='gps', location=(0, 0))
    corrected_ra = gmst.sidereal_time('mean').rad - ra
    if not LALSIMULATION:
        raise exception("lalsimulation could not be imported. please install "
                        "lalsuite to be able to use all features")
    detector = lalsim.DetectorPrefixToLALDetector(name)

    x0 = -np.cos(psi) * np.sin(corrected_ra) - \
          np.sin(psi) * np.cos(corrected_ra) * np.sin(dec)
    x1 = -np.cos(psi) * np.cos(corrected_ra) + \
          np.sin(psi) * np.sin(corrected_ra) * np.sin(dec)
    x2 =  np.sin(psi) * np.cos(dec)
    x = np.array([x0, x1, x2])
    dx = detector.response.dot(x)

    y0 =  np.sin(psi) * np.sin(corrected_ra) - \
          np.cos(psi) * np.cos(corrected_ra) * np.sin(dec)
    y1 =  np.sin(psi) * np.cos(corrected_ra) + \
          np.cos(psi) * np.sin(corrected_ra) * np.sin(dec)
    y2 =  np.cos(psi) * np.cos(dec)
    y = np.array([y0, y1, y2])
    dy = detector.response.dot(y)

    fplus = (x * dx - y * dy).sum()
    fcross = (x * dy + y * dx).sum()

    return fplus, fcross

def _waveform_plot(detectors, maxL_params, **kwargs):
    """Plot the maximum likelihood waveform for a given approximant.

    Parameters
    ----------
    detectors: list
        list of detectors that you want to generate waveforms for
    maxL_params: dict
        dictionary of maximum likelihood parameter values
    kwargs: dict
        dictionary of optional keyword arguments
    """
    logging.info("Generating the maximum likelihood waveform plot") 
    if not LALSIMULATION:
        raise exception("lalsimulation could not be imported. please install "
                        "lalsuite to be able to use all features")
    delta_frequency = kwargs.get("delta_f", 1./256)
    minimum_frequency = kwargs.get("f_min", 5.)
    maximum_frequency = kwargs.get("f_max", 1000.)
    frequency_array = np.arange(minimum_frequency, maximum_frequency,
                                delta_frequency)

    approx = lalsim.GetApproximantFromString(maxL_params["approximant"])
    mass_1 = maxL_params[b"mass_1"]*MSUN_SI
    mass_2 = maxL_params[b"mass_2"]*MSUN_SI
    luminosity_distance = maxL_params[b"luminosity_distance"]*PC_SI*10**6
    if b"phi_jl" in maxL_params.keys():
        iota, S1x, S1y, S1z, S2x, S2y, S2z = \
            lalsim.SimInspiralTransformPrecessingNewInitialConditions(
                maxL_params[b"iota"], maxL_params[b"phi_jl"], maxL_params[b"tilt_1"],
                maxL_params[b"tilt_2"], maxL_params[b"phi_12"], maxL_params[b"a_1"],
                maxL_params[b"a_2"], mass_1, mass_2, kwargs.get("f_ref", 10.),
                maxL_params[b"phase"])
    else:
        iota, S1x, S1y, S1z, S2x, S2y, S2z = maxL_params[b"iota"], 0., 0., 0., \
                                             0., 0., 0.
    phase = maxL_params[b"phase"] if b"phase" in maxL_params.keys() else 0.0
    h_plus, h_cross = lalsim.SimInspiralChooseFDWaveform(mass_1, mass_2, S1x,
                          S1y, S1z, S2x, S2y, S2z,
                          luminosity_distance, iota,
                          phase, 0.0, 0.0, 0.0, delta_frequency,
                          minimum_frequency, maximum_frequency,
                          kwargs.get("f_ref", 10.), None, approx)
    h_plus = h_plus.data.data
    h_cross = h_cross.data.data
    h_plus = h_plus[:len(frequency_array)]
    h_cross = h_cross[:len(frequency_array)]
    fig = plt.figure()
    colors=['b', 'r']
    linestyle=['-', '--']
    for num, i in enumerate(detectors):
        i = i.decode("utf-8")
        ar = __antenna_response(str(i), maxL_params[b"ra"], maxL_params[b"dec"],
                                maxL_params[b"psi"], maxL_params[b"geocent_time"])
        plt.plot(frequency_array, abs(h_plus*ar[0]+h_cross*ar[1]),
                 color=colors[num], linestyle=linestyle[num],
                 linewidth=2.0, label=i)
    plt.xscale("log")
    plt.yscale("log")
    plt.xlabel(r"Frequency $[Hz]$", fontsize=16)
    plt.ylabel(r"Strain $[1/\sqrt{Hz}]$", fontsize=16)
    plt.grid()
    plt.legend(loc="best")
    return fig

def _waveform_comparison_plot(maxL_params_list, colors, **kwargs):
    """Generate a plot which compares the maximum likelihood waveforms for
    each approximant.

    Parameters
    ----------
    maxL_params_list: list
        list of dictionaries containing the maximum likelihood parameter
        values for each approximant
    colors: list
        list of colors to be used to differentiate the different approximants
    kwargs: dict
        dictionary of optional keyword arguments
    """
    logging.info("Generating the maximum likelihood waveform comparison plot "
                 "for H1") 
    if not LALSIMULATION:
        raise Exception("LALSimulation could not be imported. Please install "
                        "LALSuite to be able to use all features")
    delta_frequency = kwargs.get("delta_f", 1./256)
    minimum_frequency = kwargs.get("f_min", 10.)
    maximum_frequency = kwargs.get("f_max", 1000.)
    frequency_array = np.arange(minimum_frequency, maximum_frequency,
                                delta_frequency)

    fig = plt.figure()
    for num, i in enumerate(maxL_params_list):
        approx = lalsim.GetApproximantFromString(i["approximant"])
        mass_1 = i[b"mass_1"]*MSUN_SI
        mass_2 = i[b"mass_2"]*MSUN_SI
        luminosity_distance = i[b"luminosity_distance"]*PC_SI*10**6
        iota, S1x, S1y, S1z, S2x, S2y, S2z = \
            lalsim.SimInspiralTransformPrecessingNewInitialConditions(
                i[b"iota"], i[b"phi_jl"], i[b"tilt_1"], i[b"tilt_2"], i[b"phi_12"],
                i[b"a_1"], i[b"a_2"], mass_1, mass_2, kwargs.get("f_ref", 10.),
                i[b"phase"])
        h_plus, h_cross = lalsim.SimInspiralChooseFDWaveform(mass_1, mass_2,
                              S1x, S1y, S1z, S2x, S2y, S2z,
                              luminosity_distance, iota, i[b"phase"], 0.0,
                              0.0, 0.0, delta_frequency, minimum_frequency,
                              maximum_frequency, kwargs.get("f_ref", 10.),
                              None, approx)
        h_plus = h_plus.data.data
        h_cross = h_cross.data.data
        h_plus = h_plus[:len(frequency_array)]
        h_cross = h_cross[:len(frequency_array)]
        ar = __antenna_response("H1", i[b"ra"], i[b"dec"], i[b"psi"],
                                i[b"geocent_time"])
        plt.plot(frequency_array, abs(h_plus*ar[0]+h_cross*ar[1]),
                 color=colors[num], label=i["approximant"], linewidth=2.0)
    plt.xscale("log")
    plt.yscale("log")
    plt.grid()
    plt.legend(loc="best")
    plt.xlabel(r"Frequency $[Hz]$", fontsize=16)
    plt.ylabel(r"Strin $[1/\sqrt{Hz}]$", fontsize=16)
    return fig

def _sky_map_plot(ra, dec, **kwargs):
    """Plot the sky location of the source for a given approximant

    Parameters
    ----------
    ra: list
        list of samples for right ascension
    dec: list
        list of samples for declination
    kwargs: dict
        optional keyword arguments
    """
    ra = [i-np.pi for i in ra]
    logging.info("Generating the sky map plot")
    fig = plt.figure()
    ax = plt.subplot(111, projection="hammer")
    ax.cla()
    ax.grid()
    ax.set_xticklabels([r"$22^{h}$", r"$20^{h}$", r"$18^{h}$", r"$16^{h}$",
                        r"$14^{h}$", r"$12^{h}$", r"$10^{h}$", r"$8^{h}$",
                        r"$6^{h}$", r"$4^{h}$", r"$2^{h}$"]) 
    levels = [1.0 - np.exp(-0.5), 1 - np.exp(-2), 1-np.exp(-9./2.)]

    H,X,Y = np.histogram2d(ra, dec, bins=50)
    H = gaussian_filter(H, kwargs.get("smooth", 0.9))
    Hflat = H.flatten()
    indicies = np.argsort(Hflat)[::-1]
    Hflat = Hflat[indicies]

    CF = np.cumsum(Hflat)
    CF /= CF[-1]

    V = np.empty(len(levels))
    for num, i in enumerate(levels):
        try:
            V[num] = Hflat[CF <= i][-1]
        except:
            V[num] = Hflat[0]
    V.sort()
    m = np.diff(V) == 0
    while np.any(m):
        V[np.where(m)[0][0]] *= 1.0 - 1e-4
        m = np.diff(V) == 0
    V.sort()
    X1, Y1 = 0.5 * (X[1:] + X[:-1]), 0.5 * (Y[1:] + Y[:-1])

    H2 = H.min() + np.zeros((H.shape[0] + 4, H.shape[1] + 4))
    H2[2:-2, 2:-2] = H
    H2[2:-2, 1] = H[:, 0]
    H2[2:-2, -2] = H[:, -1]
    H2[1, 2:-2] = H[0]
    H2[-2, 2:-2] = H[-1]
    H2[1, 1] = H[0, 0]
    H2[1, -2] = H[0, -1]
    H2[-2, 1] = H[-1, 0]
    H2[-2, -2] = H[-1, -1]
    X2 = np.concatenate([X1[0] + np.array([-2, -1]) * np.diff(X1[:2]), X1, 
                         X1[-1] + np.array([1, 2]) * np.diff(X1[-2:]),])
    Y2 = np.concatenate([Y1[0] + np.array([-2, -1]) * np.diff(Y1[:2]), Y1,
                         Y1[-1] + np.array([1, 2]) * np.diff(Y1[-2:]),])

    plt.contour(X2, Y2, H2.T, V, colors=["#AED6F1","#3498DB","#21618C"],
                linewidths=2.0)

    xticks = np.arange(-np.pi, np.pi, np.pi/6)
    ax.set_xticks(xticks)
    labels = [r"$%s^{h}$" %(np.round((i+np.pi)*3.82, 1)) for i in xticks]
    ax.set_xticklabels(labels) 
    return fig

def _sky_map_comparison_plot(ra_list, dec_list, approximants, colors, **kwargs):
    """Generate a plot that compares the sky location for multiple approximants

    Parameters
    ----------
    ra_list: 2d list
        list of samples for right ascension for each approximant
    dec_list: 2d list
        list of samples for declination for each approximant
    approximants: list
        list of approximants used to generate the samples
    colors: list
        list of colors to be used to differentiate the different approximants
    kwargs: dict
        optional keyword arguments
    """
    logging.info("Generating the sky map comparison plot")
    fig = plt.figure()
    ax = plt.subplot(111, projection="hammer")
    ax.cla()
    ax.grid()
    ax.set_xticklabels([r"$22^{h}$", r"$20^{h}$", r"$18^{h}$", r"$16^{h}$",
                        r"$14^{h}$", r"$12^{h}$", r"$10^{h}$", r"$8^{h}$",
                        r"$6^{h}$", r"$4^{h}$", r"$2^{h}$"])
    levels = [1.0 - np.exp(-0.5), 1 - np.exp(-2), 1-np.exp(-9./2.)]

    for num, i in enumerate(ra_list):
        H,X,Y = np.histogram2d(i, dec_list[num], bins=50)
        H = gaussian_filter(H, kwargs.get("smooth", 0.9))
        Hflat = H.flatten()
        indicies = np.argsort(Hflat)[::-1]
        Hflat = Hflat[indicies]

        CF = np.cumsum(Hflat)
        CF /= CF[-1]

        V = np.empty(len(levels))
        for ind, i in enumerate(levels):
            try:
                V[ind] = Hflat[CF <= i][-1]
            except:
                V[ind] = Hflat[0]
        V.sort()
        m = np.diff(V) == 0
        while np.any(m):
            V[np.where(m)[0][0]] *= 1.0 - 1e-4
            m = np.diff(V) == 0
        V.sort()
        X1, Y1 = 0.5 * (X[1:] + X[:-1]), 0.5 * (Y[1:] + Y[:-1])

        H2 = H.min() + np.zeros((H.shape[0] + 4, H.shape[1] + 4))
        H2[2:-2, 2:-2] = H
        H2[2:-2, 1] = H[:, 0]
        H2[2:-2, -2] = H[:, -1]
        H2[1, 2:-2] = H[0]
        H2[-2, 2:-2] = H[-1]
        H2[1, 1] = H[0, 0]
        H2[1, -2] = H[0, -1]
        H2[-2, 1] = H[-1, 0]
        H2[-2, -2] = H[-1, -1]
        X2 = np.concatenate([X1[0] + np.array([-2, -1]) * np.diff(X1[:2]), X1, 
                             X1[-1] + np.array([1, 2]) * np.diff(X1[-2:]),])
        Y2 = np.concatenate([Y1[0] + np.array([-2, -1]) * np.diff(Y1[:2]), Y1,
                             Y1[-1] + np.array([1, 2]) * np.diff(Y1[-2:]),])

        CS = plt.contour(X2, Y2, H2.T, V, colors=colors[num], linewidths=2.0)
        CS.collections[0].set_label(approximants[num])
    plt.legend(loc="best")
    return fig

def _make_corner_plot(samples, params, latex_labels, **kwargs):
    """Generate the corner plots for a given approximant

    Parameters
    ----------
    opts: argparse
        argument parser object to hold all information from the command line
    samples: nd list
        nd list of samples for each parameter for a given approximant
    params: list
        list of parameters associated with each element in samples
    approximant: str
        name of approximant that was used to generate the samples
    latex_labels: dict
        dictionary of latex labels for each parameter
    """
    logging.info("Generating the corner plot")
    # set the default kwargs
    default_kwargs = dict(
            bins=50, smooth=0.9, label_kwargs=dict(fontsize=16),
            title_kwargs=dict(fontsize=16), color='#0072C1',
            truth_color='tab:orange', quantiles=[0.16, 0.84],
            levels=(1 - np.exp(-0.5), 1 - np.exp(-2), 1 - np.exp(-9 / 2.)),
            plot_density=False, plot_datapoints=True, fill_contours=True,
            max_n_ticks=3)
    corner_parameters = [b"luminosity_distance", b"dec", b"a_2",
                         b"a_1", b"geocent_time", b"phi_jl", b"psi", b"ra", b"phase",
                         b"mass_2", b"mass_1", b"phi_12", b"tilt_2", b"iota",
                         b"tilt_1", b"chi_p", b"chirp_mass", b"mass_ratio",
                         b"symmetric_mass_ratio", b"total_mass", b"chi_eff"]
    xs = np.zeros([len(corner_parameters), len(samples)])
    for num, i in enumerate(corner_parameters):
        xs[num] = [j[params.index(b"%s" %(i))] for j in samples]
    default_kwargs['range'] = [1.0]*len(corner_parameters)
    default_kwargs["labels"] = [latex_labels[i] for i in corner_parameters]
    figure = corner.corner(xs.T, **default_kwargs)
    # grab the axes of the subplots
    axes = figure.get_axes()
    extent = axes[0].get_window_extent().transformed(figure.dpi_scale_trans.inverted())
    width, height = extent.width, extent.height
    width *= figure.dpi
    height *= figure.dpi 
    #np.savetxt("{}/plots/corner/{}_axes.txt".format(opts.webdir, approximant), extent)
    return figure

def __get_cutoff_indices(flow, fhigh, df, N):
    """
    Gets the indices of a frequency series at which to stop an overlap
    calculation.

    Parameters
    ----------
    flow: float
        The frequency (in Hz) of the lower index.
    fhigh: float
        The frequency (in Hz) of the upper index.
    df: float
        The frequency step (in Hz) of the frequency series.
    N: int
        The number of points in the **time** series. Can be odd
        or even.

    Returns
    -------
    kmin: int
    kmax: int
    """
    if flow:
        kmin = int(flow / df)
    else:
        kmin = 1
    if fhigh:
        kmax = int(fhigh / df )
    else:
        kmax = int((N + 1)/2.)
    return kmin,kmax

def _sky_sensitivity(network, resolution, maxL_params, **kwargs):
    """Generate the sky sensitivity for a given network

    Parameters
    ----------
    network: list
        list of detectors you want included in your sky sensitivity plot
    resolution: float
        resolution of the skymap
    maxL_params: dict
        dictionary of waveform parameters for the maximum likelihood waveform
    """
    logging.info("Generating the sky sensitivity for %s" %(network)) 
    if not LALSIMULATION:
        raise Exception("LALSimulation could not be imported. Please install "
                        "LALSuite to be able to use all features")
    delta_frequency = kwargs.get("delta_f", 1./256)
    minimum_frequency = kwargs.get("f_min", 20.)
    maximum_frequency = kwargs.get("f_max", 1000.)
    frequency_array = np.arange(minimum_frequency, maximum_frequency,
                                delta_frequency)

    approx = lalsim.GetApproximantFromString(maxL_params["approximant"])
    mass_1 = maxL_params[b"mass_1"]*MSUN_SI
    mass_2 = maxL_params[b"mass_2"]*MSUN_SI
    luminosity_distance = maxL_params[b"luminosity_distance"]*PC_SI*10**6
    iota, S1x, S1y, S1z, S2x, S2y, S2z = \
        lalsim.SimInspiralTransformPrecessingNewInitialConditions(
            maxL_params[b"iota"], maxL_params[b"phi_jl"], maxL_params[b"tilt_1"],
            maxL_params[b"tilt_2"], maxL_params[b"phi_12"], maxL_params[b"a_1"],
            maxL_params[b"a_2"], mass_1, mass_2, kwargs.get("f_ref", 10.),
            maxL_params[b"phase"])
    h_plus, h_cross = lalsim.SimInspiralChooseFDWaveform(mass_1, mass_2, S1x,
                          S1y, S1z, S2x, S2y, S2z,
                          luminosity_distance, iota,
                          maxL_params[b"phase"], 0.0, 0.0, 0.0, delta_frequency,
                          minimum_frequency, maximum_frequency,
                          kwargs.get("f_ref", 10.), None, approx)
    h_plus = h_plus.data.data
    h_cross = h_cross.data.data
    h_plus = h_plus[:len(frequency_array)]
    h_cross = h_cross[:len(frequency_array)]
    psd = {}
    psd["H1"] = psd["L1"] = np.array([lalsim.SimNoisePSDaLIGOZeroDetHighPower(i) for i in frequency_array])
    psd["V1"] = np.array([lalsim.SimNoisePSDVirgo(i) for i in frequency_array])
    kmin, kmax = __get_cutoff_indices(minimum_frequency, maximum_frequency,
                                      delta_frequency, (len(h_plus)-1)*2)
    ra = np.arange(-np.pi, np.pi, resolution)
    dec = np.arange(-np.pi, np.pi, resolution)
    X,Y = np.meshgrid(ra, dec)
    N = np.zeros([len(dec), len(ra)])
    
    indices = np.ndindex(len(ra), len(dec))
    for ind in indices:
        ar = {}
        SNR = {}
        for i in network:
            ard = __antenna_response(i, ra[ind[0]], dec[ind[1]],
                                     maxL_params[b"psi"], maxL_params[b"geocent_time"])
            ar[i] = [ard[0], ard[1]]
            strain = np.array(h_plus*ar[i][0]+h_cross*ar[i][1])
            integrand = np.conj(strain[kmin:kmax])*strain[kmin:kmax]/psd[i][kmin:kmax]
            integrand = integrand[:-1]
            SNR[i] = np.sqrt(4*delta_frequency*np.sum(integrand).real)
            ar[i][0] *= SNR[i]
            ar[i][1] *= SNR[i]
        numerator = 0.0
        denominator = 0.0
        for i in network:
            numerator += sum(i**2 for i in ar[i])
            denominator += SNR[i]**2
        N[ind[1]][ind[0]] = (((numerator/denominator)**0.5))
    fig = plt.figure()
    ax = plt.subplot(111, projection="hammer")
    ax.cla()
    ax.grid()
    plt.pcolormesh(X,Y,N)
    ax.set_xticklabels([r"$22^{h}$", r"$20^{h}$", r"$18^{h}$", r"$16^{h}$",
                        r"$14^{h}$", r"$12^{h}$", r"$10^{h}$", r"$8^{h}$",
                        r"$6^{h}$", r"$4^{h}$", r"$2^{h}$"])
    return fig



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

from distutils.core import setup
from setuptools import find_packages

with open("README.md", "r") as f:
    readme = f.read()

setup(name='pesummary',
      version='0.1.2',
      description='Python package to produce summary pages for Parameter '
                  'estimation codes',
      author='Charlie Hoy',
      author_email='charlie.hoy@ligo.org',
      url='https://git.ligo.org/charlie.hoy/pesummary',
      download_url='https://git.ligo.org/charlie.hoy/pesummary',
      install_requires=[
        'h5py',
        'numpy',
        'corner',
        'matplotlib',
        'deepdish',
        'pandas',
        'pygments',
        'astropy',
        'lalsuite',
        'pytest'],
      include_package_data=True,
      packages=find_packages(),
      package_dir={'pesummary': 'pesummary'},
      package_data={'pesummary': ['js/*.js', 'css/*.css']},
      scripts=['pesummary/bin/summarypages.py'],
      classifiers=[
          "Programming Language :: Python :: 3.6"],
      license='MIT',
      long_description=readme,
      long_description_content_type='text/markdown')



# Copyright (C) 2018  Charlie Hoy <charlie.hoy@ligo.org>
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import os

def make_dir(path):
    if os.path.isdir(path):
        pass
    else:
        os.mkdir(path)

def guess_url(web_dir, host, user):
    """Guess the base url from the host name

    Parameters
    ----------
    web_dir: str
        path to the web directory where you want the data to be saved
    host: str
        the host name of the machine where the python interpreter is currently
        executing
    user: str
        the user that is current executing the python interpreter
    """
    ligo_data_grid=False
    if 'public_html' in web_dir:
        ligo_data_grid=True
    if ligo_data_grid:
        path = web_dir.split("public_html")[1]
        if "raven" in host or "arcca" in host:
            url = "https://geo2.arcca.cf.ac.uk/~{}".format(user)
        elif "cit" in host or "caltech" in host:
            url = "https://ldas-jobs.ligo.caltech.edu/~{}".format(user)
        elif 'ligo-wa' in host:
            url = "https://ldas-jobs.ligo-wa.caltech.edu/~{}".format(user)
        elif 'uwm' in host or 'nemo' in host:
            url = "https://ldas-jobs.phys.uwm.edu/~{}".format(user)
        elif 'phy.syr.edu' in host:
            url = "https://sugar-jobs.phy.syr.edu/~{}".format(user)
        elif 'vulcan' in host:
            url = "https://galahad.aei.mpg.de/~{}".format(user)
        elif 'atlas' in host:
            url = "https://atlas1.atlas.aei.uni-hannover.de/~{}".format(user)
        elif 'iucca' in host:
            url = "https://ldas-jobs.gw.iucaa.in/~{}".format(user)
        else:
            url = "https://{}/~{}".format(host, user)
        url += path
    else:
        url = "https://{}".format(web_dir)
    return url
